package com.starbuds.server.service.task.async;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.server.service.task.dao.ScoreRankDAO;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 积分排行异步检查
 *
 * @author Lay
 * @date 2020/2/29
 */
@Component
@Slf4j
public class ScoreRankPersistenceAsync {

    @Reference(version = "1.0.0")
    CacheService cacheService;

    @Resource
    ScoreRankDAO scoreRankDAO;

    @Async
    public void checkSize(long delay, String tableName, Long size, Integer timeNum, Integer scoreType, String key) {
        try {
            Thread.sleep(delay);
        } catch (Exception e) {
            log.error("线程延迟失败", e);
        }

        // 执行检查
        int count = scoreRankDAO.countByKey(tableName, timeNum, scoreType);
        if (count == Convert.toInt(size)) {
            log.info("{} 持久化任务处理完毕，执行删除", key);
            cacheService.delKey(key);
        } else {
            log.error("{} 持久化任务处理失败，要求数量 {}，实际数量 {}", key, size, count);
        }
    }
}
