package com.starbuds.server.service.task.dao;

import com.starbuds.server.common.pojo.db.LotteryGameConfigDO;
import com.starbuds.server.common.pojo.db.LotteryGameConfigItemDO;
import com.starbuds.server.common.pojo.db.LotteryGameTimerDO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;
import org.springframework.stereotype.Repository;

/**
 * 抽奖活动
 *
 * @author aoe
 * @date 2021/11/23
 */
@Mapper
@Repository
public interface LotteryGameDAO {

    /**
     * 查询抽奖游戏配置
     *
     * @param configId
     * @return
     */
    @Select("SELECT configId,configName,playType,notice " +
            "FROM tb_lottery_game_config " +
            "WHERE configId = #{configId} " +
            "AND isDeleted = 0")
    LotteryGameConfigDO selectLotteryGameConfig(long configId);

    /**
     * 抽奖游戏配置详情
     *
     * @param configItemId
     * @return
     */
    @Select("SELECT configItemId,configId,itemName,costType,costItem,costValue,iconUrl,countDown," +
            "minDurationSeconds,maxDurationSeconds,rewardPool,clickRate " +
            "FROM tb_lottery_game_config_item " +
            "WHERE configItemId = #{configItemId} " +
            "AND isDeleted = 0 " +
            "AND status = 1"
    )
    LotteryGameConfigItemDO selectLotteryGameConfigItem(long configItemId);

    /**
     * 查询抽奖定时器配置
     *
     * @param timerId
     * @param status  LotteryGameTimerStatusEnum
     * @return
     */
    @Select("SELECT timerId,lotteryNo,playType,lotteryGameConfigItemId,totalValue,stockNum,joinNum,rewardNum," +
            "giveRewardTotal,leftTotal,STATUS,startTime,endTime " +
            "FROM tb_lottery_game_timer " +
            "WHERE timerId = #{timerId} " +
            "AND status = #{status} " +
            "AND isDeleted = 0")
    LotteryGameTimerDO selectLotteryGameTimer(long timerId, int status);

    /**
     * 更新抽奖定时器配置状态为进行中
     *
     * @param timerId
     * @param inProgress
     * @param standby
     * @return
     */
    @Update("UPDATE tb_lottery_game_timer " +
            "SET status = #{inProgress} " +
            "WHERE timerId = #{timerId} " +
            "AND status = #{standby}")
    int updateTimer2InProgress(long timerId, int inProgress, int standby);
}
