package com.starbuds.server.service.task.dao;

import com.starbuds.server.common.pojo.db.StatRoomFlowDO;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;

import com.starbuds.server.service.task.entity.rtc.StatRtcRoomDTO;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Mapper;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 房间消费统计-房间里用户消费统计 Mapper
 */
@Mapper
@Repository
public interface StatRoomFlowDAO {

    /**
     * 插入数据
     */
    int insert(StatRoomFlowDO po);

    /**
     * 插入批量数据
     */
    int insertBatch(List<StatRoomFlowDO> list);

    /**
     * 插入或更新数据
     */
    int insertOrUpdate(StatRoomFlowDO po);

    /**
     * 更新数据
     */
    int update(UpdateWrapper uw);

    /**
     * 查询一条记录
     */
    StatRoomFlowDO selectOne(QueryWrapper qw);

    /**
     * 查询所有记录
     */
    StatRoomFlowDO selectList(QueryWrapper qw);

    /**
     * 查询总数
     */
    int selectCount(QueryWrapper qw);

    /**
     * 查询列表(分页)
     */
    List<StatRoomFlowDO> selectPageList(QueryWrapper qw);

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    @MapKey("hisId")
    Map<Long, StatRoomFlowDO> selectMapByIds(@Param("ids") Collection ids);

    /**
     * 消费用户
     *
     * @param createDate
     * @return
     */
    List<StatRtcRoomDTO> selectRoomPaidByTime(@Param("createDate") Integer createDate);

    /**
     * 直刷背包人
     *
     * @param createDate
     * @return
     */
    List<StatRtcRoomDTO> selectRoomPaid(@Param("createDate") Integer createDate, @Param("isBag") Integer isBag, @Param("minAmount") Integer minAmount, @Param("maxAmount") Integer maxAmount);

    /**
     * 根据ids获得 DO Map
     *
     * @param ids room_id集合
     * @return Map<主键, DO对象>
     */
    List<StatRtcRoomDTO> selectListByRoomIds(@Param("ids") Collection ids, @Param("start") Integer start, @Param("end") Integer end, @Param("startDate") Integer startDate, @Param("endDate") Integer endDate, @Param("isBag") Integer isBag);


    /**
     * 直刷-普通流水
     *
     * @param createDate
     * @return
     */
    List<StatRtcRoomDTO> selectDirectNormalFlow(@Param("createDate") Integer createDate, @Param("isBag") Integer isBag, @Param("isLuck") Integer isLuck);

}