package com.starbuds.server.service.task.dao;

import com.starbuds.server.common.pojo.db.UserDO;
import com.starbuds.server.service.task.entity.UserRechargeDTO;
import com.starbuds.server.service.task.entity.user.UserVO;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/6/19 16:00
 * @Description 用户持久层
 */
@Mapper
@Repository
public interface UserDAO {

    /**
     * 获取用户
     *
     * @param userSexSet
     * @param areaIdSet
     * @param channelIdSet
     * @param wealthLevelSet
     * @param charmLevelSet
     * @param anchorType
     * @param isAnchor
     * @param hasRecharged
     * @param userIdentity
     * @return
     */
    List<Long> selectUsers(@Param("userSexSet") Set<Integer> userSexSet,
                           @Param("areaIdSet") Set<Integer> areaIdSet,
                           @Param("channelIdSet") Set<Long> channelIdSet,
                           @Param("wealthLevelSet") Set<Integer> wealthLevelSet,
                           @Param("charmLevelSet") Set<Integer> charmLevelSet,
                           @Param("anchorType") Integer anchorType,
                           @Param("isAnchor") Integer isAnchor,
                           @Param("hasRecharged") Integer hasRecharged,
                           @Param("userIdentity") Integer userIdentity);

    /**
     * 查询充值
     *
     * @param time
     * @param payStatus
     * @return
     */
    List<UserRechargeDTO> selectUserRecharge(@Param("time") Long time, @Param("payStatus") Integer payStatus);

    /**
     * 查询用户的渠道id
     */
    Long selectChannelId(@Param("userId") Long userId);

    /**
     * 通过身份证号查询用户ID
     *
     * @param idCardNo
     * @return
     */
    List<Long> getUserIDByIDCardNo(@Param("idCardNo") String idCardNo);

    /**
     * 获取用户No
     *
     * @param userId
     * @return
     */
    UserDO getUserByUserId(@Param("userId") Long userId);

    /**
     * 用户Map
     *
     * @param userIds 用户号Map
     * @return key：用户id value：用户信息
     */
    @MapKey("userId")
    Map<Long, UserVO> selectUserMap(@Param("userIds") Set<Long> userIds);

    /***
     * 批量查询用户
     * @param pageIndex
     * @param pageSize
     * @return
     */
    List<UserVO> selectCensorUsersList(@Param("startTimestamp") Long startTimestamp, @Param("endTimestamp") Long endTimestamp,
                                       @Param("cursor") int pageIndex, @Param("pageSize") int pageSize);

    /**
     * 批量更新审核用户记录
     *
     * @param list
     * @return
     */
    int updateCensorUserInfoRecord(List<UserDO> list);

    /**
     * 批量更新用户头像的审核
     *
     * @param list
     * @return
     */
    int updateCensorUserAvatarRecord(List<UserDO> list);


    int updateUserStatActivityLogin(@Param("statDate") Integer statDate, @Param("lotteryUserNum") Integer lotteryUserNum, @Param("lottertPrice") Integer lottertPrice, @Param("updateTime") Long updateTime);

    int insertUserStatActivityLogin(@Param("hisId") Long hisId, @Param("userNum") Integer userNum, @Param("statDate") Integer statDate, @Param("createTime") Long createTime, @Param("updateTime") Long updateTime);
}
