package com.starbuds.server.service.task.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.finance.*;
import com.starbuds.server.common.constant.task.ExportExcelBizTypeEnum;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.finance.CloudAccountOrderNotify;
import com.starbuds.server.common.pojo.finance.ThirdPartyWithdrawHisDO;
import com.starbuds.server.common.pojo.finance.UserBankcardDO;
import com.starbuds.server.common.pojo.finance.UserWalletWithdrawHisVO;
import com.starbuds.server.common.pojo.util.PageListUtil;
import com.starbuds.server.service.task.dao.*;
import com.starbuds.server.service.task.entity.admin.AdminDO;
import com.starbuds.server.service.task.entity.excel.export.UserWalletWithdrawHisExcel;
import com.starbuds.server.service.task.entity.user.UserVO;
import com.starbuds.server.service.task.job.ExportExcel;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import static com.starbuds.server.common.constant.finance.WithdrawPayTypeEnum.*;

/**
 * @author laoliu
 * @date 2021/8/25 14:53
 * @description 导出钱包提现记录
 */
@Component
public class UserWalletWithdrawExportExcelImpl implements ExportExcel {

    @Resource
    private UserWalletWithdrawHisDAO userWalletWithdrawHisDAO;

    @Resource
    private UserWalletDAO userWalletDAO;

    @Resource
    private UserDAO userDAO;

    @Resource
    private UserAuthDAO userAuthDAO;

    @Resource
    private UserBankcardDAO userBankcardDAO;

    @Resource
    private AdminDAO adminDAO;

    @Override
    public ExportExcelBizTypeEnum getBizType() {
        return ExportExcelBizTypeEnum.USER_WALLET_WITHDRAW;
    }

    @Override
    public Class getClassType() {
        return UserWalletWithdrawHisExcel.class;
    }

    @Override
    public String getFileName() {
        return "粉钻提现";
    }

    @Override
    public String getSheetName() {
        return "粉钻提现";
    }

    @Override
    public List<UserWalletWithdrawHisExcel> getRowsList(Long startTime, Long endTime) {
        List<UserWalletWithdrawHisDO> dataList = new ArrayList<>();
        Integer pageIndex = 1;
        int pageSize = 1000;
        while (pageIndex != null) {
            PageList<UserWalletWithdrawHisDO> pageList = getPageList(pageIndex, pageSize, startTime, endTime);
            pageIndex = PageListUtil.nextPage(pageList);
            List<UserWalletWithdrawHisDO> list = pageList.getList();
            if (CollectionUtils.isEmpty(list)) {
                break;
            }
            dataList.addAll(list);
        }

        if (CollectionUtils.isEmpty(dataList)) {
            return Collections.emptyList();
        }

        Set<String> walletIdSet = new HashSet<>();
        // 云账户转账记录
        Set<Long> cloudAccountSet = new HashSet<>();
        // 易纳税转账记录
        Set<Long> easyTaxSet = new HashSet<>();
        // 整合第三方打款：财鱼、云销客
        Set<Long> thirdPartyWithdrawSet = new HashSet<>();
        for (UserWalletWithdrawHisDO po : dataList) {
            if (po == null) {
                continue;
            }
            WithdrawPayTypeEnum payTypeEnum = WithdrawPayTypeEnum.toEnum(po.getPayType());
            if (payTypeEnum == null) {
                continue;
            }
            Long hisId = po.getHisId();
            if (hisId == null) {
                continue;
            }
            String walletId = po.getWalletId();
            if (walletId != null) {
                walletIdSet.add(walletId);
            }

            switch (payTypeEnum) {
                case CLOUD_ACCOUNT_TRANSFER:
                    cloudAccountSet.add(hisId);
                    break;
                case EASY_TAX_TRANSFER_1:
                case EASY_TAX_TRANSFER_2:
                case EASY_TAX_TRANSFER_3:
                case EASY_TAX_TRANSFER_4:
                    easyTaxSet.add(hisId);
                    break;
                case CAIYU_TRANSFER:
                case YUNXIAOKE_TRANSFER:
                    thirdPartyWithdrawSet.add(hisId);
                    break;
            }
        }

        Map<String, UserWalletDO> walletMap = CollectionUtil.isNotEmpty(walletIdSet) ? userWalletDAO.selectWalletMapByIds(walletIdSet) : Collections.emptyMap();
        Set<Long> walletUserIds = walletMap.values().stream().map(UserWalletDO::getUserId).collect(Collectors.toSet());
        Map<Long, UserVO> userMap = CollectionUtil.isNotEmpty(walletUserIds) ? userDAO.selectUserMap(walletUserIds) : Collections.EMPTY_MAP;
        Map<Long, UserAuthDO> authMap = CollectionUtil.isNotEmpty(walletUserIds) ? userAuthDAO.selectAuthMapByIds(walletUserIds) : Collections.EMPTY_MAP;

        Map<String, Long> walletUserMap = new HashMap<>();
        walletMap.forEach((k, v) -> walletUserMap.put(k, Optional.ofNullable(v).map(UserWalletDO::getUserId).orElse(null)));
        // 银行卡
        List<UserBankcardDO> bankcardParams = new ArrayList<>();
        dataList.forEach(e -> {
            UserBankcardDO param = new UserBankcardDO();
            param.setUserId(walletUserMap.get(e.getWalletId()));
            param.setCardNo(e.getCardNo());
            bankcardParams.add(param);
        });
        Map<Long, UserBankcardDO> bankcardMap = userBankcardDAO.selectUserMobileMap(bankcardParams);
        Set<Long> adminIds = dataList.stream().map(UserWalletWithdrawHisDO::getApplyAdminId).collect(Collectors.toSet());
        Map<Long, AdminDO> adminMap = CollectionUtil.isNotEmpty(adminIds) ? adminDAO.selectAdminMap(adminIds) : Collections.EMPTY_MAP;

        Map<Long, CloudAccountTransferHisDO> cloudAccountMap = CollectionUtils.isEmpty(cloudAccountSet) ?
                Collections.emptyMap() : userWalletWithdrawHisDAO.selectCloudAccountTransferHisMapByBizIds(cloudAccountSet);
        Map<Long, EasyTaxTransferHisDO> easyTaxMap = CollectionUtils.isEmpty(easyTaxSet) ?
                Collections.emptyMap() : userWalletWithdrawHisDAO.selectEasyTaxTransferHisMapByBizIds(easyTaxSet);
        Map<Long, ThirdPartyWithdrawHisDO> thirdPartyWithdrawMap = CollectionUtils.isEmpty(thirdPartyWithdrawSet) ?
                Collections.emptyMap() : userWalletWithdrawHisDAO.selectThirdPartyWithdrawHisMapByBizIds(thirdPartyWithdrawSet);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        return dataList.stream()
                .filter(Objects::nonNull)
                .filter(po -> po.getHisId() != null)
                .filter(po -> WithdrawPayTypeEnum.toEnum(po.getPayType()) != null)
                .map(vo -> {
                    UserWalletWithdrawHisExcel excel = new UserWalletWithdrawHisExcel();
                    Long hisId = vo.getHisId();

                    UserWalletDO userWallet = CollectionUtil.isNotEmpty(walletMap) ? walletMap.get(vo.getWalletId()) : null;
                    if (userWallet != null) {
                        Long userId = userWallet.getUserId();
                        UserVO user = CollectionUtil.isNotEmpty(userMap) ? userMap.get(userId) : null;
                        excel.setUserName(Optional.ofNullable(user).map(UserVO::getUserName).orElse(""));
                        excel.setUserNo(user.getUserNo() == null ? "" : user.getUserNo() + "");
                        excel.setWithdrawAccount(vo.getCardNo());

                        UserAuthDO userAuth = authMap.get(userId);
                        excel.setRealName(Optional.ofNullable(userAuth).map(UserAuthDO::getRealName).orElse(""));
                        excel.setIdCardNo(Optional.ofNullable(userAuth).map(UserAuthDO::getIdCardNo).orElse(""));

                        UserBankcardDO bankcard = bankcardMap.get(userId);
                        excel.setMobile(Optional.ofNullable(bankcard).map(UserBankcardDO::getMobile).orElse(""));

                        String walletTypeName = "";
                        if (userWallet.getType() != null) {
                            WalletTypeEnum walletTypeEnum = WalletTypeEnum.toEnum(userWallet.getType());
                            if (walletTypeEnum != null) {
                                walletTypeName = walletTypeEnum.getName();
                            }
                        }
                        excel.setAmount(Convert.toStr(vo.getAmount() + "\n" + walletTypeName));

                        excel.setActualAmount(vo.getActualAmount() == null ? "-" : Convert.toStr(vo.getActualAmount().setScale(2, RoundingMode.HALF_UP)));
                        excel.setAgentAmount(vo.getAgentAmount() == null ? "-" : Convert.toStr(vo.getAgentAmount().setScale(2, RoundingMode.HALF_UP)));
                        excel.setTaxAmount(vo.getTaxAmount() == null ? "-" : Convert.toStr(vo.getTaxAmount().setScale(2, RoundingMode.HALF_UP)));
                        excel.setAgentRatio(vo.getAgentRatio() == null ? "-" : Convert.toStr(vo.getAgentRatio()));
                        excel.setUserRatio(vo.getUserRatio() == null ? "-" : Convert.toStr(vo.getUserRatio()));
                        excel.setUserTaxRatio(vo.getUserTaxRatio() == null ? "-" : Convert.toStr(vo.getUserTaxRatio()));

                        if (vo.getApplyStatus() != null) {
                            WalletWithdrawHisApplyStatus walletWithdrawHisApplyStatus = WalletWithdrawHisApplyStatus.toEnum(vo.getApplyStatus());
                            if (walletWithdrawHisApplyStatus != null) {
                                excel.setApplyStatusName(walletWithdrawHisApplyStatus.getName());
                            }
                        }

                        excel.setPayTypeName(vo.getPayType() != null ? WithdrawPayTypeEnum.toEnum(vo.getPayType()).getName() : "");

                        StringBuilder auditInfo = new StringBuilder("提现时间：").append(sdf.format(vo.getHisCreateTime()));
                        if (vo.getHisApplyTime() != null) {
                            auditInfo.append("\n" + "审核时间：" + sdf.format(vo.getHisApplyTime()));
                        }
                        AdminDO adminDO = CollectionUtil.isNotEmpty(adminMap) ? adminMap.get(vo.getApplyAdminId()) : null;
                        String applyAdminName = Optional.ofNullable(adminDO).map(AdminDO::getAdminName).orElse("");
                        if (StringUtils.isNotEmpty(applyAdminName)) {
                            auditInfo.append("\n" + "审核人：" + applyAdminName);
                        }
                        if (StringUtils.isNotEmpty(vo.getApplyRemarks())) {
                            auditInfo.append("\n" + "审核备注：" + vo.getApplyRemarks());
                        }
                        excel.setAuditInfo(auditInfo.toString());
                    }

                    // 打款状态
                    WithdrawPayTypeEnum payTypeEnum = WithdrawPayTypeEnum.toEnum(vo.getPayType());
                    switch (payTypeEnum) {
                        case CLOUD_ACCOUNT_TRANSFER:
                            CloudAccountTransferHisDO cloudAccount = cloudAccountMap.get(hisId);
                            if (cloudAccount != null) {
                                excel.setPayStatusStr(PayStatusEnum.toEnum(cloudAccount.getPayStatus()).getName());
                            }
                            break;
                        case EASY_TAX_TRANSFER_1:
                        case EASY_TAX_TRANSFER_2:
                        case EASY_TAX_TRANSFER_3:
                        case EASY_TAX_TRANSFER_4:
                            EasyTaxTransferHisDO easyTax = easyTaxMap.get(hisId);
                            if (easyTax != null) {
                                excel.setPayStatusStr(PayStatusEnum.toEnum(easyTax.getPayStatus()).getName());
                            }
                            break;
                        case CAIYU_TRANSFER:
                        case YUNXIAOKE_TRANSFER:
                            ThirdPartyWithdrawHisDO thirdParty = thirdPartyWithdrawMap.get(hisId);
                            if (thirdParty != null) {
                                ThirdPartyWithdrawStatusEnum thirdPartyWithdrawStatusEnum = ThirdPartyWithdrawStatusEnum.toEnum(thirdParty.getWithdrawStatus());
                                if (thirdPartyWithdrawStatusEnum != null) {
                                    switch (thirdPartyWithdrawStatusEnum) {
                                        case HAS_BEEN_SENT:
                                            excel.setPayStatusStr(PayStatusEnum.WAITTING.getName());
                                            break;
                                        case CALLBACK_FAIL:
                                            excel.setPayStatusStr(PayStatusEnum.FAILED.getName());
                                            break;
                                        case CALLBACK_SUSSES:
                                            excel.setPayStatusStr(PayStatusEnum.SUCCESSED.getName());
                                            break;
                                    }
                                }
                            }
                            break;
                        default:
                            excel.setPayStatusStr("-");
                            break;
                    }

                    return excel;
                })
                .collect(Collectors.toList());
    }

    private PageList<UserWalletWithdrawHisDO> getPageList(int pageIndex, int pageSize, Long startTime, Long endTime) {
        Integer refType = RefTypeEnum.USER_WITHDRAW.getValue();
        PageList<UserWalletWithdrawHisDO> page = PageList.ofBigPageSize(pageIndex, pageSize);
        Integer total = userWalletWithdrawHisDAO.getUserWalletWithdrawPageListCount(startTime, endTime, refType);
        page.setTotal(total);
        List<UserWalletWithdrawHisDO> list = userWalletWithdrawHisDAO.getUserWalletWithdrawPageList(page.getPageCursor(), pageSize, startTime, endTime, refType);
        page.setList(list);
        return page;
    }
}
