package com.starbuds.server.service.task.impl.sailtob;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.finance.ThirdPartyCompanyEnum;
import com.starbuds.server.common.service.data.ReleaseConfigService;
import com.starbuds.server.service.task.dao.UserSignContractDAO;
import com.starbuds.server.service.task.entity.user.UserSignContractDO;
import com.starbuds.server.service.task.util.HttpUtil;
import com.starbuds.server.service.task.util.SailtobSignUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;


/***
 * @author qinjiang
 * @description 智企行打款服务
 * @date 2021/10/22
 */
@Service
@Slf4j
public class SailtobApiService {
    static String signType = "RSA";
    static String encType = "DES";
    static String charset = "UTF-8";

    //    static SailtobConfig sailtobConfig = null;
    //税优地余额,使用的时候按照 多-->少 的顺序排列
    static Map<String, SailtobWithholdingAccount> sailtobWithholdingMap = new HashMap<>(5);

    @Resource
    UserSignContractDAO userSignContractDAO;

    @Reference(version = "1.0.0")
    private ReleaseConfigService releaseConfigService;

    /**
     * 签约人才
     *
     * @param name
     * @param cardNo
     * @param idCard
     * @param mobile
     * @param idCardFront
     * @param idCardBack
     * @param remark
     * @param bussinessID
     * @return
     */
    public boolean signSoho(String name, String cardNo, String mobile, String idCard, String idCardFront, String idCardBack, String remark, Long bussinessID) {
        SailtobConfig sailtobConfig = getSailtobConfig();
        if (sailtobConfig == null) {
            return false;
        }

        SailtobTax sailtobTax = getMatchSailtobTax(bussinessID);
        if (sailtobTax == null) {
            return false;
        }

        byte[] idCardFrontStream = getFileStream(idCardFront);
        byte[] idCardBackStream = getFileStream(idCardBack);

        String desKey = sailtobConfig.getDesKey();
        Map<String, Object> params = new HashMap<>();
        params.put("name", SailtobSignUtil.encrypt(name, desKey));
        params.put("cardNo", SailtobSignUtil.encrypt(cardNo, desKey));
        params.put("idCard", SailtobSignUtil.encrypt(idCard, desKey));
        params.put("mobile", SailtobSignUtil.encrypt(mobile, desKey));
        params.put("levyCode", sailtobTax.getLevyCode());
        params.put("idCardType", "A");
        params.put("remark1", remark);

        Map<String, Object> attachParam = new HashMap<>();
        attachParam.put("idCardFront", bytesToHexString(idCardFrontStream));
        attachParam.put("idCardBack", bytesToHexString(idCardBackStream));

        String strHost = String.format("%s/soho/singleSoho/%s", sailtobConfig.getHost(), sailtobConfig.getTenantCode());
        String resData = sendPOST(strHost, params, attachParam, sailtobConfig);
        log.info("智企云：人才签约：：请求地址：" + strHost);
        log.info("智企云：人才签约：：请求参数：" + JSON.toJSONString(params));
        log.info("智企云：人才签约：：响应文本：" + resData);
        if (StringUtils.isEmpty(resData)) {
            log.info("智企云：人才签约响应文本错误");
            return false;
        }

        // {"reqId":"1635246211091","resCode":"6016","resMsg":"该用户信息已经做过签约","sign":"lWWv4Wu+CGHpqSpPNc37Z8Zv9mfmwJjW8gneOQTDBS0vyMbJPeUgiHL508QPLPw/1eQWHy/KaYNhUDALiw+VFRDbyfgmMEa7LOiMU4E44YSj6/AtY7Jtul52qExWtPlNc2jl7A60NOdsucwqVKrEOYOCpeMcOZuAznX5jQefiEM=","ip":"125.119.235.186","remark1":"我是标签","remark2":null}

        //{"ip":"47.99.137.164","sign":"a5ZwoqqitaKNlezd6EfntR3N05HqQrSfCwEqN4kzWpCtGSSNFFYHK5cmnJ4E5dX498+qQCg8r3QbdomHXoQ8LQagQKJaeGTwpe1LQ6TCNkQACR5vNzvU1ZSKwulVNOyKh1yaX4nJnqcBUKB5QLjJLt6YoUk5/DN+mB8BJ9y2ttw=","resCode":"0000","resMsg":"成功","remark1":"9294409891907","reqId":"1636430715914"}
        JSONObject jsonObject = JSON.parseObject(resData);
        log.info("智企云：人才签约：解析文本：" + jsonObject.toJSONString());
        if (jsonObject.containsKey("resCode")) {
            String resCode = jsonObject.getString("resCode");
            // 6016:用户信息已经做过签约
            if ("6016".equals(resCode)) {
                return true;
            }
            if (!"0000".equals(resCode)) {
                return false;
            }
        }

        Map<String, String> signMap = new HashMap();
        signMap.put("remark1", jsonObject.getString("remark1"));
        signMap.put("reqId", jsonObject.getString("reqId"));

        String strSign = SailtobSignUtil.sortKey(signMap);
        boolean isCorrect = SailtobSignUtil.verify(strSign, jsonObject.getString("sign"), sailtobConfig.getPublicKey());
        if (!isCorrect) {
            log.info("智企云：人才签约数据校验失败");
            return false;
        }

//        String data = jsonObject.getString("data");
//        String dataString = new String(SailtobSignUtil.decode(data),charset);

        return true;
    }

    //查询人才
    public boolean querySoho(String name, String cardNo, String idCard, String mobile, String remark, SailtobConfig sailtobConfig, SailtobTax sailtobTax) throws Exception {
        Map<String, Object> params = new HashMap<>();
        params.put("name", SailtobSignUtil.encrypt(name, sailtobConfig.getDesKey()));
        params.put("cardNo", SailtobSignUtil.encrypt(cardNo, sailtobConfig.getDesKey()));
        params.put("idCard", SailtobSignUtil.encrypt(idCard, sailtobConfig.getDesKey()));
        params.put("mobile", SailtobSignUtil.encrypt(mobile, sailtobConfig.getDesKey()));
//        params.put("levyCode", sailtobConfig.levyCode);
        params.put("remark1", remark);

        String strHost = String.format("%s/soho/querySoho/%s", sailtobConfig.getHost(), sailtobConfig.getTenantCode());
        String resData = sendPOST(strHost, params, null, sailtobConfig);
        if (resData == null) {
            return false;
        }

        JSONObject jsonObject = JSON.parseObject(resData);
        if (jsonObject.containsKey("resCode")) {
            String resCode = jsonObject.getString("resCode");
            if (!"0000".equals(resCode)) {
                return false;
            }
        }

//        {"reqId":"1635400226070","resCode":"0000","resMsg":"成功","sign":"H9jXIEYuNdpUEHUIq71+7yeOkX1tYbHz+ZAm3z8UI7tUV3PdoD3mAJSl8HAkUsjW93g7gqo5tfVwbSCBv+WZiyd2kQSBlUI994KPmqwXg0EDTqe2+zknoHsxHlydLn+w/Gjhz7VUCXWlCpWvigGgrIMx+Xn6ZoZ25GNcLJQ4bRo=","ip":null,"name":"kC4qh/Dh6alS6nDCWtMpbQ==","idCard":"W5rfPy1qMRgA4t7N8nVJSi6M3gT5ehbl","bankCard":"0G2xrdGYg6V5YJ0DyvfQ49BgItMGy3M4","mobile":"pJ3VTR5pElbsR0nPcQlTkA==","levyCode":"1000030412","state":"SIGN"}

        String sign = jsonObject.getString("sign");
        jsonObject.remove("sign");
        jsonObject.remove("resCode");
        jsonObject.remove("resMsg");
        jsonObject.remove("ip");

        Map<String, String> signMap = JSON.parseObject(jsonObject.toJSONString(), Map.class);
        String strSign = SailtobSignUtil.sortKey(signMap);
        boolean isCorrect = verify(signMap, sign);
        if (!isCorrect) {
            return false;
        }

        //state  SIGN:已签  NOSIGN:未签
        return "sign".equals(jsonObject.getString("state"));
    }

    /**
     * 发送请求
     *
     * @param strURL
     * @param param       传递参数(会签名)
     * @param attachParam 附加参数(不参与签名)
     * @return
     * @throws Exception
     */
    private String sendPOST(String strURL, Map<String, Object> param, Map<String, Object> attachParam, SailtobConfig sailtobConfig) {
        String reqId = String.valueOf(System.currentTimeMillis());

        Map<String, Object> commonParam = new TreeMap();
        commonParam.put("version", sailtobConfig.getVersion());
        commonParam.put("reqId", reqId);
        commonParam.put("merId", sailtobConfig.getMerId());
        commonParam.putAll(param);

        String linkString = SailtobSignUtil.createLinkString(commonParam, false);
        String signString = SailtobSignUtil.sign(linkString, sailtobConfig.getPrivateKey());
        commonParam.put("sign", signString);
        commonParam.put("signType", signType);
        commonParam.put("encType", encType);

        if (attachParam != null && attachParam.keySet().size() > 0) {
            commonParam.putAll(attachParam);
        }

        String strJSON = JSON.toJSONString(commonParam);
        log.info("智企行 {}  参数：{}", strURL, strJSON);

        String res = null;
        try {
            res = HttpUtil.post(strURL, strJSON);
            log.info("智企行 {} 请求成功:{}", strURL, res);
        } catch (Exception e) {
            log.error("智企行 {} 请求失败:{}", strURL, e.toString());
        }

        return res;
    }

    /**
     * 初始智企行配置
     *
     * @throws Exception
     */
    private SailtobConfig getSailtobConfig() {
        String strSailtobConfig = releaseConfigService.getCachReleaseConfig("sailtob");
        if (StringUtils.isEmpty(strSailtobConfig)) {
            log.info("获取在线打款智企行配置失败");
            return null;
        }

        return JSON.parseObject(strSailtobConfig, SailtobConfig.class);
    }

    /**
     * 查询用户是否签名
     *
     * @param userId
     * @param bussinessId
     * @return
     */
    public boolean isUserSign(Long userId, Long bussinessId) {
        SailtobConfig sailtobConfig = getSailtobConfig();
        if (sailtobConfig == null) {
            return false;
        }

        SailtobTax sailtobTax = getMatchSailtobTax(bussinessId);
        if (sailtobTax == null) {
            return false;
        }

        UserSignContractDO userSignContractDO = userSignContractDAO.selectUserSignRecord(userId, ThirdPartyCompanyEnum.SAILTOB.getId(), Long.valueOf(sailtobTax.getLevyCode()));
        if (userSignContractDO == null) {
            return false;
        }

        return true;
    }

    /**
     * 验签
     *
     * @param signMap map
     * @param sign    sign字符串
     * @return
     */
    public boolean verify(Map<String, String> signMap, String sign) {
        if (signMap == null) {
            return false;
        }

        SailtobConfig sailtobConfig = getSailtobConfig();
        if (sailtobConfig == null) {
            return false;
        }

        String strSign = SailtobSignUtil.sortKey(signMap);
        return SailtobSignUtil.verify(strSign, sign, sailtobConfig.getPublicKey());
    }

    /**
     * 查找配置中的税优地
     *
     * @param bussinessID
     * @return
     */
    public SailtobTax getMatchSailtobTax(Long bussinessID) {
        SailtobConfig sailtobConfig = getSailtobConfig();
        if (sailtobConfig == null) {
            return null;
        }

        SailtobTax sailtobTax = null;
        List<SailtobTax> listTax = sailtobConfig.getTax();
        for (SailtobTax tax : listTax) {
            if (tax.getId() == bussinessID.intValue()) {
                sailtobTax = tax;
                break;
            }
        }

        return sailtobTax;
    }

    /**
     * 得到文件流
     *
     * @param url 图片地址
     * @return
     */
    public static byte[] getFileStream(String url) {
        try {
            URL httpUrl = new URL(url);
            HttpURLConnection conn = (HttpURLConnection) httpUrl.openConnection();
            conn.setRequestMethod("GET");
            conn.setConnectTimeout(10 * 1000);
            InputStream inStream = conn.getInputStream();//通过输入流获取图片数据
            byte[] btImg = readInputStream(inStream);//得到图片的二进制数据
            return btImg;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 从输入流中获取数据
     *
     * @param inStream 输入流
     * @return
     * @throws Exception
     */
    public static byte[] readInputStream(InputStream inStream) throws Exception {
        ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        byte[] buffer = new byte[1024];
        int len = 0;
        while ((len = inStream.read(buffer)) != -1) {
            outStream.write(buffer, 0, len);
        }
        inStream.close();
        return outStream.toByteArray();
    }

    public static String bytesToHexString(byte[] src) {
        StringBuilder stringBuilder = new StringBuilder("");
        if (src == null || src.length <= 0) {
            return null;
        }
        for (byte aSrc : src) {
            int v = aSrc & 0xFF;
            String hv = Integer.toHexString(v);
            if (hv.length() < 2) {
                stringBuilder.append(0);
            }
            stringBuilder.append(hv);
        }
        return stringBuilder.toString();
    }
}
