package com.starbuds.server.service.task.job;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.pojo.activity.vo.ActivityUserLoginStatVO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.ReleaseConfigService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.ActivityLotteryHisRewardsDAO;
import com.starbuds.server.service.task.dao.UserDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;

/**
 * @author: ruomu
 * @date: 2021/12/8 created
 */
@Component
@Slf4j
public class ActivityLoginUserStatJob {

    @Resource
    UserDAO userDAO;

    @Resource
    ActivityLotteryHisRewardsDAO lotteryHisRewardsDAO;

    @Reference(version = "1.0.0")
    ReleaseConfigService releaseConfigService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @XxlJob(value = JobHandlers.ACTIVITY_LOGIN_USER_STAT)
    @Transactional(rollbackFor = Exception.class)
    public ReturnT<String> activityLoginUserHandler(String param) {

        String config = releaseConfigService.getCachReleaseConfig("loginActivity2022");

        if (StringUtils.isNotEmpty(config)) {
            JSONObject obj = JSON.parseObject(config);
            String activityLotteryId = obj.getString("activityLotteryId");

            //默认统计前一天的数据
            long startTime = DateUtil.getYesterday(new Date()).getTime();
            long endTime = DateUtil.getDayBegin();
            int statDate = DateUtil.getIntDate(DateUtil.getYesterday(new Date()));

            //如果传入统计时间
            if (StringUtils.isNotEmpty(param)) {
                JSONObject p = JSON.parseObject(param);
                startTime = p.getLong("startTime");
                endTime = p.getLong("endTime");
                statDate = p.getInteger("statDate");
            }

            //总金额和抽奖人数
            ActivityUserLoginStatVO stat = lotteryHisRewardsDAO.selectActivityLotteryHisRewardsList(activityLotteryId, startTime, endTime);
            long timeNow = System.currentTimeMillis();
            int price = stat.getPrice() == null ? 0 : stat.getPrice();
            int userNum = stat.getUserNum() == null ? 0 : stat.getUserNum();


            if (userDAO.updateUserStatActivityLogin(statDate, userNum, price, System.currentTimeMillis()) <= 0) {
                userDAO.insertUserStatActivityLogin(identityService.getId(), userNum, statDate, timeNow, timeNow);
            }

        }

        return ReturnT.SUCCESS;
    }
}
