package com.starbuds.server.service.task.job;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.pojo.db.AnchorAttendanceDO;
import com.starbuds.server.common.pojo.media.LiveStatVO;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.AnchorAttendanceDAO;
import com.starbuds.server.service.task.dao.LiveHisDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/6/2 15:38
 * @Description 主播考勤
 */
@Slf4j
@Component
public class AnchorAttendanceJob {

    @Resource
    AnchorAttendanceDAO anchorAttendanceDAO;

    @Resource
    LiveHisDAO liveHisDAO;

    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.ANCHOR_ATTENDANCE)
    public ReturnT<String> anchorAttendanceHandler(String param) {
        log.info("执行主播考勤 {}", param);
        long curTime = System.currentTimeMillis();
        try {
            Long startTime = 0L;
            Long endTime = 0L;
            String periodStr = "";
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd HH:mm:ss");
            SimpleDateFormat yyyyMMdd = new SimpleDateFormat("yyyyMMdd");
            Calendar instance = Calendar.getInstance();
            if (StringUtils.isNotEmpty(param)) {

                String startStr = param + " 00:00:00";
                startTime = sdf.parse(startStr).getTime();
                String endStr = param + " 23:59:59";
                endTime = sdf.parse(endStr).getTime();

                instance.setTime(new Date(startTime));

                int year = instance.get(Calendar.YEAR);
                int month = instance.get(Calendar.MONTH) + 1;
                int day = instance.get(Calendar.DAY_OF_MONTH);
                int periodNum = 1;
                if (day > 15) {
                    periodNum = 2;
                }
                periodStr = String.format("%d-%d-%d", year, month, periodNum);
            } else {
                // 昨天
                instance.add(Calendar.DAY_OF_YEAR, -1);
                int year = instance.get(Calendar.YEAR);
                int month = instance.get(Calendar.MONTH) + 1;
                int day = instance.get(Calendar.DAY_OF_MONTH);
                int periodNum = 1;
                if (day > 15) {
                    periodNum = 2;
                }
                periodStr = String.format("%d-%d-%d", year, month, periodNum);

                // 当天凌晨0点0分
                instance.set(Calendar.HOUR_OF_DAY, 0);
                instance.set(Calendar.MINUTE, 0);
                instance.set(Calendar.SECOND, 0);
                instance.set(Calendar.MILLISECOND, 0);
                startTime = instance.getTimeInMillis();

                // 当天凌晨23点59分
                instance.set(Calendar.HOUR_OF_DAY, 23);
                instance.set(Calendar.MINUTE, 59);
                instance.set(Calendar.SECOND, 59);
                instance.set(Calendar.MILLISECOND, 999);
                endTime = instance.getTimeInMillis();
            }


            List<AnchorAttendanceDO> list = new ArrayList<>();

            log.info("periodStr:{},startTime:{},endTime:{}", periodStr, startTime, endTime);

            List<LiveStatVO> liveList = liveHisDAO.selectLiveStat(startTime, endTime);

            if (CollectionUtils.isEmpty(liveList)) {
                log.info("主播考勤没有数据执行成功");
                return ReturnT.SUCCESS;
            }

            for (LiveStatVO statVO : liveList) {
                AnchorAttendanceDO attendance = new AnchorAttendanceDO();
                attendance.setUserId(statVO.getUserId());
                attendance.setLiveOnlineDuration(statVO.getLiveDurationSum());
                attendance.setLiveEffectiveDuration(statVO.getLiveEffectiveDurationSum());
                attendance.setLiveNum(statVO.getLiveTimes());
                attendance.setLiveCoinIncome(statVO.getLiveCoinIncomeSum());
                attendance.setSingleChatOnlineDuration(0L);
                attendance.setPeriodStr(periodStr);
                attendance.setCreateTime(curTime);
                attendance.setDate(Convert.toLong(yyyyMMdd.format(startTime)));
                list.add(attendance);
            }

            anchorAttendanceDAO.batchInsert(list);
        } catch (Exception e) {
            log.info("主播考勤执行失败", e);
            return ReturnT.FAIL;
        }

        log.info("主播考勤执行成功");
        return ReturnT.SUCCESS;
    }

}
