package com.starbuds.server.service.task.job;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.constant.activity.ActivityRankTypeEnum;
import com.starbuds.server.common.constant.activity.ActivityTypeEnum;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.pojo.activity.bo.AnnualGalaPeakFamilyRankBO;
import com.starbuds.server.common.pojo.db.ActivityAnnualAwardConfigDO;
import com.starbuds.server.common.pojo.db.ActivityAnnualAwardItemDO;
import com.starbuds.server.common.pojo.db.UserActivityScoreDO;
import com.starbuds.server.common.pojo.message.im.activity.AnnualGalaFinishMsg;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.pojo.rtc.RtcRoomListItem;
import com.starbuds.server.common.service.activity.ActivityAnniversaryService;
import com.starbuds.server.common.service.activity.ActivityAnnualAwardConfigService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author laoliu
 * @date 2021/12/20 15:37
 * @description 巅峰家族活动奖励发放
 * 跑任务时，参数传入activityId
 */
@Component
@Slf4j
public class AnnualGalaPeakFamilyRewardJob {

    @Reference(version = "1.0.0")
    ActivityAnnualAwardConfigService activityAnnualAwardConfigService;

    @Reference(version = "1.0.0")
    ActivityAnniversaryService activityAnniversaryService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @XxlJob(value = JobHandlers.ANNUAL_GALA_PEAK_FAMILY_FINISH_REWARD)
    @Transactional(rollbackFor = Exception.class)
    public ReturnT<String> annualGalaPeakFamilyFinishRewardHandler(String param) throws ParseException {

        if (param == null) {
            return new ReturnT<>("activityId必传");
        }

        Long activityId = Long.valueOf(param);
        log.info("巅峰家族活动发放奖励: 活动activityId: {}", activityId);

        // 0奖励
        AnnualGalaFinishMsg finishRewardMsg = new AnnualGalaFinishMsg();
        finishRewardMsg.setActivityType(ActivityTypeEnum.PEAKFAMILY.getValue());
        finishRewardMsg.setMsgType(0);
        finishRewardMsg.setActivityId(activityId);

        List<AnnualGalaPeakFamilyRankBO> rankList = activityAnniversaryService.getPeakFamilyRankList(activityId);
        if (rankList == null) {
            log.info("巅峰家族发放奖励：rankList为空");
            return new ReturnT<>();
        }

        List<AnnualGalaPeakFamilyRankBO> rankSortedList = rankList.stream().sorted(Comparator.comparing(AnnualGalaPeakFamilyRankBO::getScore).reversed()).collect(Collectors.toList());
        List<AnnualGalaPeakFamilyRankBO> rank5List = rankSortedList.subList(0, Math.min(rankList.size(), 5));

        for (int i = 1; i < rank5List.size() + 1; i++) {

            AnnualGalaPeakFamilyRankBO val = rank5List.get(i - 1);
            // 获取满足条件中奖的人
            List<UserActivityScoreDO> peakFamilyRewardUserList = activityAnniversaryService.getPeakFamilyRewardUserList(activityId, val.getFamilyId());

            // 获取当前排名的奖励信息
            log.info("满足当前家族条件的人:familyId:{}, peakFamilyRewardUserList:{}", val.getFamilyId(), JSON.toJSONString(peakFamilyRewardUserList));
            log.info("开始获取当前排名的奖励信息activityId:{}, ActivityRankTypeEnum:{}", activityId, ActivityRankTypeEnum.TOTAL.getValue());
            Map<Integer, ActivityAnnualAwardConfigDO> awardConfigDOMap = activityAnniversaryService.getRewardConfig(activityId, ActivityRankTypeEnum.TOTAL.getValue());
            log.info("获取当前排名的奖励信息:awardConfigDOMap:{}", JSON.toJSONString(awardConfigDOMap));
            finishRewardMsg.setImgUrl(awardConfigDOMap.get(i).getAlertStyle());
            // 为中奖者发放奖品并且发送通知
            for (int j = 0; j < peakFamilyRewardUserList.size(); j++) {

                UserActivityScoreDO v = peakFamilyRewardUserList.get(j);
                List<ActivityAnnualAwardItemDO> rewardItems = activityAnniversaryService.doGivePeakFamilyUserReward(i, v.getUserId(), activityId, awardConfigDOMap);
                log.info("巅峰家族发放奖励成功,userId:{}, familyId: {}, activityId: {}, familyRank: {}, rewardItems: {}", v.getUserId(), val.getFamilyId(), activityId, i, JSON.toJSONString(rewardItems));
                if (rewardItems.size() > 0) {
                    // 发奖励成功
                    // 所有中奖者弹框
                    RtcRoomListItem userStayRoom = rtcRoomService.getUserStayRoom(v.getUserId());

                    if (StringUtils.isBlank(awardConfigDOMap.get(i).getAlertStyle())) {
                        continue;
                    }

                    if (userStayRoom != null) {
                        finishRewardMsg.setUserId(v.getUserId());
                        imService.sendRtcRoomMsg(ImTargetType.GROUP, null, v.getUserId(), userStayRoom.getRoomId(), finishRewardMsg);
                        log.info("巅峰家族发放奖励消息弹窗发送成功,userId:{}, familyId: {}, activityId: {}, familyRank: {}, rewardItems: {}, finishMsg: {}， userStayRoom.getRoomId(): {}", v.getUserId(), val.getFamilyId(), activityId, i, JSON.toJSONString(rewardItems), JSON.toJSONString(finishRewardMsg), userStayRoom.getRoomId());
                    }

                    // 所有中奖者系统消息
                    SystemMsg systemMsg = new SystemMsg();
                    systemMsg.setContentType(com.starbuds.server.common.constant.message.im.ContentType.TEXT);
                    systemMsg.setTitle("周年庆活动");
                    List<String> rewardTxt = rewardItems.stream().map(e -> String.format("%s*%d", e.getName(), e.getNum())).collect(Collectors.toList());
                    String body = String.format("恭喜获得「巅峰家族」奖励, %s", StringUtils.join(rewardTxt, ","));
                    systemMsg.setBody(body);
                    systemMsg.setActionType(ImActionType.NO_ACTION.getValue());
                    imService.sendCustomImMsg(null, v.getUserId(), systemMsg, null);
                    log.info("巅峰家族发放奖励系统消息发送成功,userId:{}, familyId: {}, activityId: {}, familyRank: {}, rewardItems: {}, systemMsg: {}", v.getUserId(), val.getFamilyId(), activityId, i, JSON.toJSONString(rewardItems), JSON.toJSONString(systemMsg));
                } else {
                    log.error("巅峰家族发放奖励失败,userId:{}, familyId: {}, activityId: {}, familyRank: {}, awardConfigDOMap: {}", v.getUserId(), val.getFamilyId(), activityId, i, JSON.toJSONString(awardConfigDOMap));
                }

            }
        }

        return new ReturnT<>("成功");
    }
}
