package com.starbuds.server.service.task.job;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.statistics.StatPlatformTypeEnum;
import com.starbuds.server.common.constant.statistics.StatTimeType;
import com.starbuds.server.common.pojo.db.ActivityLotteryHisDO;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.ActivityLotteryHisDAO;
import com.starbuds.server.service.task.dao.StatPlatformDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.util.Strings;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * ` * 修复活动统计数据(传入活动id初冬暖阳)
 *
 * @author: ruomu
 * @date: 2022/1/4 created
 */
@Slf4j
@Component
public class FixStatPlatformJob {

    @Resource
    ActivityLotteryHisDAO activityLotteryHisDAO;
    @Resource
    StatPlatformDAO statPlatformDAO;

    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.FIX_STAT_PLATFROM)
    public ReturnT<String> fixStatPlatfromHandler(String param) {

        if (StringUtils.isEmpty(param)) {
            return ReturnT.FAIL;
        }

        long activityId = Long.parseLong(param);
        List<ActivityLotteryHisDO> list = activityLotteryHisDAO.getListByActivityId(activityId);

        list.stream().forEach(r -> {
            long time = r.getHisCreateTime();
            long year = getTime(time, StatTimeType.YEAR);
            long year_month = getTime(time, StatTimeType.YEAR_MONTH);
            long year_month_day = getTime(time, StatTimeType.YEAR_MONTH_DAY);
            long year_month_day_hour = getTime(time, StatTimeType.YEAR_MONTH_DAY_HOUR);
            long year_month_day_hour_minute = getTime(time, StatTimeType.YEAR_MONTH_DAY_HOUR_MINUTE);
            long channelId = r.getChannelId();

            //修复抽奖活动消耗金币
            statPlatformDAO.updateStat(year, StatTimeType.YEAR, channelId, BigDecimal.valueOf(r.getHisCostAmount()), StatPlatformTypeEnum.LUCKY_ACTIVITY_PAY_AMOUNT.getValue());
            statPlatformDAO.updateStat(year_month, StatTimeType.YEAR_MONTH, channelId, BigDecimal.valueOf(r.getHisCostAmount()), StatPlatformTypeEnum.LUCKY_ACTIVITY_PAY_AMOUNT.getValue());
            statPlatformDAO.updateStat(year_month_day, StatTimeType.YEAR_MONTH_DAY, channelId, BigDecimal.valueOf(r.getHisCostAmount()), StatPlatformTypeEnum.LUCKY_ACTIVITY_PAY_AMOUNT.getValue());
            statPlatformDAO.updateStat(year_month_day_hour, StatTimeType.YEAR_MONTH_DAY_HOUR, channelId, BigDecimal.valueOf(r.getHisCostAmount()), StatPlatformTypeEnum.LUCKY_ACTIVITY_PAY_AMOUNT.getValue());
            statPlatformDAO.updateStat(year_month_day_hour_minute, StatTimeType.YEAR_MONTH_DAY_HOUR_MINUTE, channelId, BigDecimal.valueOf(r.getHisCostAmount()), StatPlatformTypeEnum.LUCKY_ACTIVITY_PAY_AMOUNT.getValue());

            //修复抽奖活动中奖价值
            statPlatformDAO.updateStat(year, StatTimeType.YEAR, channelId, BigDecimal.valueOf(r.getHisRewardValue()), StatPlatformTypeEnum.LUCKY_ACTIVITY_WIN_AMOUNT.getValue());
            statPlatformDAO.updateStat(year_month, StatTimeType.YEAR_MONTH, channelId, BigDecimal.valueOf(r.getHisRewardValue()), StatPlatformTypeEnum.LUCKY_ACTIVITY_WIN_AMOUNT.getValue());
            statPlatformDAO.updateStat(year_month_day, StatTimeType.YEAR_MONTH_DAY, channelId, BigDecimal.valueOf(r.getHisRewardValue()), StatPlatformTypeEnum.LUCKY_ACTIVITY_WIN_AMOUNT.getValue());
            statPlatformDAO.updateStat(year_month_day_hour, StatTimeType.YEAR_MONTH_DAY_HOUR, channelId, BigDecimal.valueOf(r.getHisRewardValue()), StatPlatformTypeEnum.LUCKY_ACTIVITY_WIN_AMOUNT.getValue());
            statPlatformDAO.updateStat(year_month_day_hour_minute, StatTimeType.YEAR_MONTH_DAY_HOUR_MINUTE, channelId, BigDecimal.valueOf(r.getHisRewardValue()), StatPlatformTypeEnum.LUCKY_ACTIVITY_WIN_AMOUNT.getValue());


        });

        return ReturnT.SUCCESS;
    }

    private long getTime(long timestamp, Integer statTimeType) {
        Date date = new Date(timestamp);
        SimpleDateFormat sdf = new SimpleDateFormat(getTimeTypeFormat(statTimeType));
        return Convert.toLong(sdf.format(date));
    }

    private static String getTimeTypeFormat(Integer statTimeType) {

        switch (statTimeType) {
            case StatTimeType.YEAR:
                return "yyyy";
            case StatTimeType.YEAR_MONTH:
                return "yyyyMM";
            case StatTimeType.YEAR_MONTH_DAY:
                return "yyyyMMdd";
            case StatTimeType.YEAR_MONTH_DAY_HOUR:
                return "yyyyMMddHH";
            case StatTimeType.YEAR_MONTH_DAY_HOUR_MINUTE:
                return "yyyyMMddHHmm";
            case StatTimeType.YEAR_MONTH_DAY_HOUR_MINUTE_SECOND:
                return "yyyyMMddHHmmss";
            default:
                return Strings.EMPTY;
        }
    }
}
