package com.starbuds.server.service.task.job;

import com.starbuds.server.common.pojo.db.ImAppInfoDO;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.ImAppInfoDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * @Author jiangxiaowei
 * @DateTime 2020-04-30 11:12:01
 * @Description im运营数据统计                                                                                                                                                                                                                                                                                         任务
 */
@Component
@Slf4j
public class ImAppInfoJob {

    @Reference(version = "1.0.0")
    ImService imService;

    @Resource
    ImAppInfoDAO imAppInfoDAO;

    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.IM_APP_INFO)
    public ReturnT<String> imAppInfoJobHandler(String param) {
        log.info("触发im运营统计 {}", param);

        List<ImAppInfoDO> appInfos = imService.getAppInfo();

        if (CollectionUtils.isEmpty(appInfos)) {
            return ReturnT.SUCCESS;
        }

        int count = 0;
        long time = System.currentTimeMillis();
        for (ImAppInfoDO appInfo : appInfos) {
            appInfo.setCreateTime(time);
            count += imAppInfoDAO.insertOrUpdate(appInfo);
            log.info("触发im运营统计 insertOrUpdate count={}", count);
        }

        if (count > 0) {
            return ReturnT.SUCCESS;
        }

        log.warn("触发im运营统计没有数据更新 {}", param);
        return ReturnT.FAIL;
    }
}
