package com.starbuds.server.service.task.job;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.pojo.message.im.rtc.RtcHourRankStatusChangedMsg;
import com.starbuds.server.common.pojo.rtc.RtcRankService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

/**
 * @Author jiangxiaowei
 * @DateTime 2021/6/17 14:54
 * @Description 小时榜更新
 */
@Slf4j
@Component
public class RtcHourRankChangeJob {

    @Reference(version = "1.0.0")
    RtcRankService rtcRankService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    ImService imService;

    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.RTC_HOUR_RANK_CHANGE)
    public ReturnT<String> rtcHourRankChangeHandler(String param) {
        Integer status = rtcRoomService.getRtcHourRankStatus();
        if (status == 1) {
            RtcHourRankStatusChangedMsg msg = new RtcHourRankStatusChangedMsg();
            msg.setStatus(status);
            msg.setList(rtcRankService.getRtcHourRank(Convert.toLong(LocalDateTime.now().minusHours(1).format(DateTimeFormatter.ofPattern("yyyyMMddHH")))));
            imService.sendRtcRoomMsg(ImTargetType.ALL, null, null, null, msg);
        }
        return ReturnT.SUCCESS;
    }

}
