package com.starbuds.server.service.task.job;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.RandomUtil;
import com.starbuds.server.common.constant.statistics.StatTimeType;
import com.starbuds.server.common.service.statistics.StatPlatformService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xxl.job.core.util.ShardingUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.Calendar;

/**
 * @Author jiangxiaowei
 * @DateTime 2020-04-30 11:12:01
 * @Description 渠道统计                                                                                                                                                                                                                                                                                         任务
 */
@Component
@Slf4j
public class StatChannelJob {

    @Reference(version = "1.0.0")
    StatPlatformService statPlatformService;

    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.STAT_CHANNEL)
    public ReturnT<String> statChannelJobHandler(String param) {
        log.info("触发渠道统计 {}", param);

        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHH");
        Long startTime = null;
        Long endTime = null;
        if (StringUtils.isNotBlank(param)) {
            try {
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(sdf.parse(param));
                calendar.add(Calendar.HOUR_OF_DAY, -1);
                startTime = Convert.toLong(sdf.format(calendar.getTime()));
            } catch (Exception e) {
                log.info("转换出错，{}", e);
            }
            // 有传参取传参的，到一小时前
            endTime = Convert.toLong(param);
        } else {
            // 否则处理一小时前的数据
            Calendar calendar = Calendar.getInstance();
            endTime = Convert.toLong(sdf.format(calendar.getTime()));
            calendar.add(Calendar.HOUR_OF_DAY, -1);
            startTime = Convert.toLong(sdf.format(calendar.getTime()));
        }

        log.info("startTime:{},endTime:{}", startTime, endTime);

        if (startTime != null && endTime != null) {

            ShardingUtil.ShardingVO shardingVO = ShardingUtil.getShardingVo();

            // 随机获取一台
            int index = RandomUtil.randomInt(0, shardingVO.getTotal());
            if (index == shardingVO.getIndex()) {
                log.info("开始调用rpc同步渠道统计,startTime:{},endTime:{}", startTime, endTime);
                statPlatformService.syncDataToChannel(startTime, endTime, StatTimeType.YEAR_MONTH_DAY_HOUR);
                log.info("渠道统计执行完成");
                return ReturnT.SUCCESS;
            }
        }
        log.info("渠道统计失败");
        return ReturnT.FAIL;
    }
}
