package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.pojo.db.GiftDO;
import com.starbuds.server.service.task.constant.PlatformGathers;
import com.starbuds.server.service.task.dao.StatPlatformGatherDAO;
import com.starbuds.server.service.task.entity.StatPlatformGatherDO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.*;
import java.util.concurrent.*;

@Slf4j
/**
 * 统计合算任务基类
 * @author qingjiang
 */
public class StatGatherJobBase {
    /** 线程池 */
    //protected static ExecutorService executorService = Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors());
    /*** 分页处理记录大小 */
    protected static int pageSize = 50000;
    /**
     * 礼物价值类型
     */
    protected static Long priceType = Long.valueOf(WalletTypeEnum.STAR_DIAMOND.getValue());
    /**
     * 导入参数
     */
    protected static String importParam = "1";

    @Autowired
    StatPlatformGatherDAO statPlatformGatherDAO;

    /**
     * 日志前缀
     */
    protected String logPrefix = "";

    protected void startGatherJobEntry(String param, Long taskType, Long ext) {
        /** 明确限定扩展值 */
        if (!taskType.equals(PlatformGathers.GATHER_Manager_COIN)) {
            ext = 0L;
        }

        Map<Long, GiftDO> giftDoMap = statPlatformGatherDAO.selectGiftId(priceType);
        List<Long> triggerList = statPlatformGatherDAO.selectTriggerId();

        /*** 确认当前批量处理的开始时间
         * 逻辑：如果有填入参数(1)则全量进行批量导入
         * */
        if (StringUtils.isBlank(param)) {
            /*** 默认轮询取最近1小时 */
            Long endTimestamp = getTimestamp(System.currentTimeMillis(), PlatformGathers.SCALE_HOUR);
            Long startTimestamp = endTimestamp - 3600 * 1000;
            log.info(logPrefix + "批量处理数据记录 createTime >= " + startTimestamp + "  and createTime < " + endTimestamp);

            Long startDayTimestamp = getTimestamp(startTimestamp, PlatformGathers.SCALE_DAY);
            Long endDayTimestamp = getTimestamp(endTimestamp, PlatformGathers.SCALE_DAY);

            if (startDayTimestamp.equals(endDayTimestamp)) {
                taskHandler(startTimestamp, endTimestamp, taskType, ext, giftDoMap, triggerList, true);
            } else {
                //转点数据维护：擦除前一天的小时维度数据，重新修正
                eraseScaleHourRecord(startDayTimestamp, endDayTimestamp, taskType, ext);
                taskHandler(startDayTimestamp, endDayTimestamp, taskType, ext, giftDoMap, triggerList, true);
                correctStatRecord(startDayTimestamp, endDayTimestamp, taskType, ext);
            }

            return;
        }
        if (!importParam.equals(param)) {
            log.info(logPrefix + "可执行参数错误：" + param);
            return;
        }

        //从2020年11月开始算，并以1个月为周期进行执行
        Calendar gatherCalendar = Calendar.getInstance();
        gatherCalendar.setTime(DateUtil.parse("202011", "yyyyMM"));

        ExecutorService executorService = Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors());
        boolean isBootExport = true;
        while (isBootExport) {
            Long startTimestamp = gatherCalendar.getTimeInMillis();
            /** 当前月的下一个月，为了获取月初始值 */
            gatherCalendar.add(Calendar.MONTH, 1);
            Long endTimestamp = gatherCalendar.getTimeInMillis();
            /*** 终止时间：默认轮询取最近1小时 */
            if (endTimestamp > System.currentTimeMillis()) {
                endTimestamp = getTimestamp(System.currentTimeMillis(), PlatformGathers.SCALE_HOUR) - 3600 * 1000;
                isBootExport = false;
            }

            Long finalEndTimestamp = endTimestamp;
            Long finalExt = ext;
            executorService.submit(new Runnable() {
                @Override
                public void run() {
                    taskHandler(startTimestamp, finalEndTimestamp, taskType, finalExt, giftDoMap, triggerList, false);
                }
            });
        }
        executorService.shutdown();
        while (true) {
            if (executorService.isTerminated()) {
                break;
            }

            try {
                Thread.sleep(1000);
            } catch (Exception e) {
                log.info(logPrefix + " 数据导入处理异常：" + e.getMessage());
            }
        }

        statYearRecord(taskType, ext);
    }

    /***
     * 擦除时间段为小时的数据
     * @param startTimestamp
     * @param endTimestamp
     * @param taskType
     * @param ext
     */
    protected void eraseScaleHourRecord(Long startTimestamp, Long endTimestamp, Long taskType, Long ext) {
        statPlatformGatherDAO.eraseScaleHourRecord(startTimestamp, endTimestamp, taskType, PlatformGathers.SCALE_HOUR, ext);
    }

    /***
     * 修正统计记录 （转点后进行数据修正）
     * @param taskType
     * @param ext
     */
    protected void correctStatRecord(Long startDayTimestamp, Long endDayTimestamp, Long taskType, Long ext) {
        Long dayQuantiy = statPlatformGatherDAO.selectSumQuantity(taskType, PlatformGathers.SCALE_HOUR, ext, startDayTimestamp, endDayTimestamp);
        statPlatformGatherDAO.updateGatherQuantity(dayQuantiy, taskType, PlatformGathers.SCALE_DAY, startDayTimestamp, ext);

        Long monthTimestamp = getTimestamp(startDayTimestamp, PlatformGathers.SCALE_MONTH);
        Long monthQuantiy = statPlatformGatherDAO.selectSumQuantity(taskType, PlatformGathers.SCALE_DAY, ext, startDayTimestamp, endDayTimestamp);
        statPlatformGatherDAO.updateGatherQuantity(monthQuantiy, taskType, PlatformGathers.SCALE_MONTH, monthTimestamp, ext);

        Long yearTimestamp = getTimestamp(startDayTimestamp, PlatformGathers.SCALE_YEAR);
        Long yearQuantiy = statPlatformGatherDAO.selectSumQuantity(taskType, PlatformGathers.SCALE_MONTH, ext, startDayTimestamp, endDayTimestamp);
        statPlatformGatherDAO.updateGatherQuantity(yearQuantiy, taskType, PlatformGathers.SCALE_YEAR, yearTimestamp, ext);
    }

    /***
     * 统计年度数据
     */
    protected void statYearRecord(Long taskType, Long ext) {
        /*** 批量插入数值后，更新年度数值记录 */
        List<String> listSql = new ArrayList<>();
        Calendar yearCalendar = Calendar.getInstance();
        yearCalendar.setTime(DateUtil.parse("2020", "yyyy"));
        boolean isContinueUpdate = true;
        while (isContinueUpdate) {
            Long startTimestamp = yearCalendar.getTimeInMillis();
            yearCalendar.add(Calendar.YEAR, 1);
            Long endTimestamp = yearCalendar.getTimeInMillis();

            /*** 终止时间：默认轮询取最近1小时 */
            if (endTimestamp > System.currentTimeMillis()) {
                endTimestamp = getTimestamp(System.currentTimeMillis(), PlatformGathers.SCALE_HOUR) - 3600 * 1000;
                isContinueUpdate = false;
            }

            //查询年度值
            Long yearQuantity = statPlatformGatherDAO.selectSumQuantity(taskType, PlatformGathers.SCALE_MONTH, ext, startTimestamp, endTimestamp);
            if (yearCalendar == null) {
                continue;
            }

            StatPlatformGatherDO yearGatherDO = new StatPlatformGatherDO();
            yearGatherDO.setType(taskType);
            yearGatherDO.setScale(PlatformGathers.SCALE_YEAR);
            yearGatherDO.setExt(ext);
            yearGatherDO.setQuantity(yearQuantity);
            yearGatherDO.setTimestamp(startTimestamp);

            listSql.addAll(eachGenerateInsertSql(Arrays.asList(yearGatherDO)));
        }

        statPlatformGatherDAO.insertBatch(listSql);
    }

    /***
     * 任务执行操作
     * @param startTimestamp 大于等于 的开始时间
     * @param endTimestamp 小于 结束时间
     * @param taskType 任务执行类型 例如：PlatformGathers.GATHER_xxx_xxx 等
     * @param ext 扩展数据 活动类型为PlatformGathers.GATHER_Manager_COIN 时，1：派币  0：扣币
     * @param isLoop 是否轮询执行（true:轮询  false:首次批量执行一次）
     */
    protected void taskHandler(Long startTimestamp, Long endTimestamp, Long taskType, Long ext, Map<Long, GiftDO> matchGiftMap, List<Long> matchTriggerList, boolean isLoop) {
        int pageIndex = 1;
        Long recordCount = 0L;
        Long beginTime = System.currentTimeMillis();

        while (true) {
            List<StatPlatformGatherDO> gatherRecords = new ArrayList<>();
            if (taskType.equals(PlatformGathers.GATHER_Trigger_GIFT)) {
                ext = 0L;
                gatherRecords = statPlatformGatherDAO.selectTriggerGiveGift(startTimestamp, endTimestamp, (pageIndex - 1) * pageSize, pageSize);
            } else if (taskType.equals(PlatformGathers.GATHER_Trigger_COIN)) {
                ext = 0L;
                gatherRecords = statPlatformGatherDAO.selectTriggerGiveCoin(priceType, startTimestamp, endTimestamp, (pageIndex - 1) * pageSize, pageSize);
            } else if (taskType.equals(PlatformGathers.GATHER_Manager_COIN)) {
                gatherRecords = statPlatformGatherDAO.selectManageGiveCoin(priceType, ext, startTimestamp, endTimestamp, (pageIndex - 1) * pageSize, pageSize);
            }

            if (CollectionUtil.isEmpty(gatherRecords) || gatherRecords.size() <= 0) {
                break;
            }

            Map<Long, StatPlatformGatherDO> mapYear = new HashMap<>(2);
            Map<Long, StatPlatformGatherDO> mapMonth = new HashMap<>(12);
            Map<Long, StatPlatformGatherDO> mapDay = new HashMap<>(31);
            Map<Long, StatPlatformGatherDO> mapHour = new HashMap<>(24);

            // 遍历对象进行确认时间段数据
            for (StatPlatformGatherDO gather : gatherRecords) {
                if (taskType.equals(PlatformGathers.GATHER_Trigger_GIFT) || taskType.equals(PlatformGathers.GATHER_Trigger_COIN)) {
                    if (!matchTriggerList.contains(gather.getOrderId())) {
                        continue;
                    }
                    if (taskType.equals(PlatformGathers.GATHER_Trigger_GIFT)) {
                        GiftDO giftDO = matchGiftMap.get(gather.getItemId());

                        if (giftDO == null) {
                            continue;
                        }
                        gather.setQuantity(giftDO.getGiftPrice() * gather.getQuantity());
                    }
                }

                Long giftTimestamp = gather.getTimestamp();
                Long giftQuantity = gather.getQuantity();
                // 确认记录对象所处的时间节点年、月、日、时
                Long yearTimestamp = getTimestamp(giftTimestamp, PlatformGathers.SCALE_YEAR);
                Long monthTimestamp = getTimestamp(giftTimestamp, PlatformGathers.SCALE_MONTH);
                Long dayTimestamp = getTimestamp(giftTimestamp, PlatformGathers.SCALE_DAY);
                Long hourTimestamp = getTimestamp(giftTimestamp, PlatformGathers.SCALE_HOUR);
                //当前分析记录的统计记录对象
                StatPlatformGatherDO yearGatherDO = matchGenerateGatherDO(mapYear.get(yearTimestamp), taskType, PlatformGathers.SCALE_YEAR, giftQuantity, yearTimestamp, ext);
                StatPlatformGatherDO monthGatherDO = matchGenerateGatherDO(mapMonth.get(monthTimestamp), taskType, PlatformGathers.SCALE_MONTH, giftQuantity, monthTimestamp, ext);
                StatPlatformGatherDO dayGatherDO = matchGenerateGatherDO(mapDay.get(dayTimestamp), taskType, PlatformGathers.SCALE_DAY, giftQuantity, dayTimestamp, ext);
                StatPlatformGatherDO hourGatherDO = matchGenerateGatherDO(mapHour.get(hourTimestamp), taskType, PlatformGathers.SCALE_HOUR, giftQuantity, hourTimestamp, ext);
                // 相应匹配的数据字典存储
                mapYear.put(yearTimestamp, yearGatherDO);
                mapMonth.put(monthTimestamp, monthGatherDO);
                mapDay.put(dayTimestamp, dayGatherDO);
                mapHour.put(hourTimestamp, hourGatherDO);
            }
            // 目标更新字典获取所有元素值的转化
            List<StatPlatformGatherDO> listMonthDO = new ArrayList<>(mapMonth.values());
            List<StatPlatformGatherDO> listDayDO = new ArrayList<>(mapDay.values());
            List<StatPlatformGatherDO> listHourDO = new ArrayList<>(mapHour.values());
            if (listHourDO.size() <= 0) {
                break;
            }

            recordCount += gatherRecords.size();
            List<String> listSql = margeGenerateSqlList(listMonthDO, listDayDO, listHourDO);
            if (isLoop) {
                List<StatPlatformGatherDO> listYearDO = new ArrayList<>(mapYear.values());
                sortPlatformGatherDO(listYearDO);
                firstElementExistToUpdate(listYearDO);
                listSql.addAll(eachGenerateInsertSql(listYearDO));
            }

            statPlatformGatherDAO.insertBatch(listSql);
            pageIndex++;
        }

        Long endTime = System.currentTimeMillis();
        log.info(logPrefix + "完成批量数据导入处理,共耗时：" + (endTime - beginTime) + "毫秒  处理：" + recordCount + "条数据！");
    }

    protected List<String> margeGenerateSqlList(List listMonthDo, List listDayDo, List listHourDo) {
        /*** 对列表进行排序，以便后续元素取值进行判断是否插入或更新 */
        sortPlatformGatherDO(listMonthDo);
        sortPlatformGatherDO(listDayDo);
        sortPlatformGatherDO(listHourDo);

        /*** 对首元素进行查询是否需要更新 */
        firstElementExistToUpdate(listMonthDo);
        firstElementExistToUpdate(listDayDo);
        firstElementExistToUpdate(listHourDo);

        /*** 批量生成SQL插入语句 */
        List<String> listMonthSql = eachGenerateInsertSql(listMonthDo);
        List<String> listDaySql = eachGenerateInsertSql(listDayDo);
        List<String> listHourSql = eachGenerateInsertSql(listHourDo);

        /*** 合并列表 */
        List<String> listSql = new ArrayList<>();
        listSql.addAll(listMonthSql);
        listSql.addAll(listDaySql);
        listSql.addAll(listHourSql);

        return listSql;
    }

    /***
     * 列表首个元素查询是否需要更新
     * @param list
     */
    protected void firstElementExistToUpdate(List<StatPlatformGatherDO> list) {
        StatPlatformGatherDO gatherDo = list.get(0);
        boolean isExistScaleYear = statPlatformGatherDAO.countExistGatherRecord(gatherDo.getType(), gatherDo.getScale(), gatherDo.getTimestamp(), gatherDo.getExt()) > 0;
        if (isExistScaleYear) {
            boolean updated = statPlatformGatherDAO.updateGatherQuantity(gatherDo.getQuantity(), gatherDo.getType(), gatherDo.getScale(), gatherDo.getTimestamp(), gatherDo.getExt()) > 0;
            if (updated) {
                list.remove(0);
            }
        }
    }

    /***
     * 按照平台汇总列表时间戳升序排序
     * @param list
     */
    protected void sortPlatformGatherDO(List<StatPlatformGatherDO> list) {
        Collections.sort(list, new Comparator<StatPlatformGatherDO>() {
            @Override
            public int compare(StatPlatformGatherDO gatherDo0, StatPlatformGatherDO gatherDo1) {
                return gatherDo0.getTimestamp().compareTo(gatherDo1.getTimestamp());
            }
        });
    }

    /***
     * 匹配生成对象
     * @param gatherDO
     * @param scale
     * @param quantity
     * @param timestamp
     * @return
     */
    protected StatPlatformGatherDO matchGenerateGatherDO(StatPlatformGatherDO gatherDO, Long type, Long scale, Long quantity, Long timestamp, Long ext) {
        if (gatherDO == null) {
            gatherDO = new StatPlatformGatherDO();
            gatherDO.setType(type);
            gatherDO.setScale(scale);
            gatherDO.setExt(ext);
            gatherDO.setQuantity(quantity);
            gatherDO.setTimestamp(timestamp);
        } else {
            gatherDO.setQuantity(gatherDO.getQuantity() + quantity);
        }

        return gatherDO;
    }

    /***
     * 轮询生成插入的SQL语句
     * @param listDo
     * @return
     */
    protected List<String> eachGenerateInsertSql(List<StatPlatformGatherDO> listDo) {
        List<String> listSql = new ArrayList<>();
        String sqlFormat = "INSERT INTO tb_stat_platform_gather(quantity,type,scale,timestamp,ext) values (%d,%d,%d,%d,%d)";

        listDo.forEach(gatherDO -> {
            String sql = String.format(sqlFormat, gatherDO.getQuantity(), gatherDO.getType(), gatherDO.getScale(), gatherDO.getTimestamp(), gatherDO.getExt());
            listSql.add(sql);
        });

        return listSql;
    }

    /***
     * 获取转换的timestamp类型
     * @param timestamp
     * @param type 4:年  3:月  2:日   1:时
     * @return
     */
    protected static Long getTimestamp(Long timestamp, Long type) {
        TimeZone chinaZone = TimeZone.getTimeZone("GMT+08:00");

        Calendar calendar = Calendar.getInstance(chinaZone);
        calendar.setTimeInMillis(timestamp);

        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);

        if (type.equals(PlatformGathers.SCALE_YEAR)) {
            calendar.set(Calendar.MONTH, 0);
            calendar.set(Calendar.DAY_OF_MONTH, 1);
            calendar.set(Calendar.HOUR_OF_DAY, 0);
        } else if (type.equals(PlatformGathers.SCALE_MONTH)) {
            calendar.set(Calendar.DAY_OF_MONTH, 1);
            calendar.set(Calendar.HOUR_OF_DAY, 0);
        } else if (type.equals(PlatformGathers.SCALE_DAY)) {
            calendar.set(Calendar.HOUR_OF_DAY, 0);
        }

        return calendar.getTimeInMillis();
    }

}
