package com.starbuds.server.service.task.job;

import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.provider.StatGoldCoinProvider;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneOffset;

/**
 * 金币统计
 *
 * @author aoe
 * @date 2021/9/13
 */
@Slf4j
@Component
public class StatGoldCoinJob {

    @Resource
    private StatGoldCoinProvider statGoldCoinProvider;

    /**
     * 金币统计
     *
     * @param param
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.STAT_GOLD_COIN_TASK)
    public ReturnT<String> statGoldCoinHandler(String param) {
        log.info("执行金币统计开始 param = {}", param);
        LocalDate date = LocalDate.now().minusDays(1);
        if (StringUtils.isNotBlank(param)) {
            date = DateUtil.toLocalDate(param);
        }
        if (date == null) {
            ReturnT<String> fail = ReturnT.FAIL;
            fail.setMsg(String.format("param=%s,不能转换为日期", param));
            return fail;
        }

        boolean statGoldCoin = statGoldCoinProvider.statByDate(date);
        if (Boolean.TRUE.equals(statGoldCoin)) {
            log.info("执行金币统计结束-成功 param={} date={}", param, date);
            return ReturnT.SUCCESS;
        }

        log.info("执行金币统计结束-失败 param={} date={}", param, date);
        return ReturnT.FAIL;
    }

    /**
     * 金币统计重建任务
     *
     * @param param
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.STAT_GOLD_COIN_REBUILD_TASK)
    public ReturnT<String> statGoldCoinRebuildHandler(String param) {
        Long beginTime = 0L;
        if (StringUtils.isNotBlank(param)) {
            beginTime = Long.valueOf(param);
        } else {
            ReturnT res = ReturnT.FAIL;
            res.setMsg("需要传入参数开始时间戳");
            return res;
        }
        Long now = System.currentTimeMillis();
        Long yesterday = now - DateUtil.DAY_OF_MILLION;
        while (beginTime < yesterday) {
            statGoldCoinProvider.statByDate(Instant.ofEpochMilli(beginTime).atZone(ZoneOffset.ofHours(8)).toLocalDate());
            beginTime = beginTime + DateUtil.DAY_OF_MILLION;
        }

        return ReturnT.SUCCESS;
    }

}
