package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.cache.RedisUserKeys;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.db.GradeConfigDO;
import com.starbuds.server.common.pojo.db.RtcRoomDO;
import com.starbuds.server.common.pojo.db.RtcRoomJoinHisDO;
import com.starbuds.server.common.pojo.db.StatRtcRoomGradeDO;
import com.starbuds.server.common.pojo.task.vo.GradeVO;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.RedisService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.common.utils.LogUtil;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.*;
import com.starbuds.server.service.task.entity.rtc.StatRtcRoomDTO;
import com.starbuds.server.service.task.entity.rtc.UserOnlineDTO;
import com.starbuds.server.service.task.entity.stat.StatBarrageRsp;
import com.starbuds.server.service.task.util.HttpUtil;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.starbuds.server.service.task.util.SpringContextUtil.isDev;

/**
 * @author laoliu
 * @date 2021/9/2 15:37
 * @description 房间消费统计
 */
@Component
@Slf4j
public class StatRtcRoomGradeJob {

    public static List<GradeVO> gradeVOS = new ArrayList<>();

    static {
        GradeVO vo1 = new GradeVO();
        GradeVO vo2 = new GradeVO();
        GradeVO vo3 = new GradeVO();
        GradeVO vo4 = new GradeVO();
        GradeVO vo5 = new GradeVO();
        vo1.setStart(0);
        vo1.setEnd(100);

        vo2.setStart(100);
        vo2.setEnd(500);

        vo3.setStart(500);
        vo3.setEnd(1000);

        vo4.setStart(1000);
        vo4.setEnd(3000);

        vo5.setStart(3000);
        vo5.setEnd(null);

        gradeVOS.add(vo1);
        gradeVOS.add(vo2);
        gradeVOS.add(vo3);
        gradeVOS.add(vo4);
        gradeVOS.add(vo5);
    }

    /**
     * 融云统计地址
     */
    @Value("${apiHost.apiNotifyRongcloud}")
    private String apiNotifyRongcloud;

    @Resource
    RtcRoomDAO rtcRoomDAO;

    @Resource
    GradeConfigDAO gradeConfigDAO;

    @Resource
    StatRtcRoomGradeDAO statRtcRoomGradeDAO;

    @Resource
    RtcRoomJoinHisDAO rtcRoomJoinHisDAO;

    @Resource
    RtcRoomFavDAO rtcRoomFavDAO;

    @Resource
    StatRtcRoomDAO statRtcRoomDAO;

    @Resource
    StatRoomFlowDAO statRoomFlowDAO;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    RedisService redisService;

    @XxlJob(value = JobHandlers.STAT_RTC_ROOM_GRADE_TASK)
    @Transactional(rollbackFor = Exception.class)
    public ReturnT<String> statRtcRoomGradeHandler(String param) throws ParseException {
        log.info("房间消费统计: {}", param);
        // 昨天日期
        SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMdd");
        Date yestDay = param != null && !"".equals(param) ? formatter.parse(param) : DateUtil.getYesterday(new Date());

        // 昨天时间YYYYMMDD
        int yestDayDate = DateUtil.getIntDate(yestDay);
        // 昨天开始时间戳
        long yestDayStartTime = DateUtil.getDayBegin(yestDay);
        long yestDayEndTime = DateUtil.getDayEnd(yestDay);
        long curTime = System.currentTimeMillis();

        log.info("房间消费统计：yestDayStartTime：" + yestDayStartTime);
        log.info("房间消费统计：yestDayEndTime：" + yestDayEndTime);

        QueryWrapper qu = new QueryWrapper();
        qu.eq("createDate", yestDayDate);
        qu.ne("roomId", 0L);
//        if (statRtcRoomGradeDAO.selectCount(qu) > 0) {
//            log.info("已执行过, 不重复执行: {}", param);
//            return new ReturnT<>("已执行过, 不重复执行");
//        }

        // 房间活跃用户
        long activityUserCurTime = System.currentTimeMillis();
        List<StatRtcRoomDTO> activityUser = rtcRoomJoinHisDAO.selectRoomJoinHisByTime(yestDayStartTime, yestDayEndTime);
        long activityUserEndTime = System.currentTimeMillis();
        String activityUserUseTime = String.format("房间活跃用户查询用时%d", activityUserEndTime - activityUserCurTime);

        // 收藏用户
        long favoriteUserCurTime = System.currentTimeMillis();
        List<StatRtcRoomDTO> favoriteUser = rtcRoomFavDAO.selectRoomFavoriteByTime(yestDayStartTime, yestDayEndTime);
        long favoriteUserEndTime = System.currentTimeMillis();
        String favoriteUserUseTime = String.format(".收藏用户查询用时%d", favoriteUserEndTime - favoriteUserCurTime);

        // 上麦用户
        long sitSeatUserCurTime = System.currentTimeMillis();
        List<StatRtcRoomDTO> sitSeatUser = statRtcRoomDAO.selectSitSeatList(yestDayDate);
        long sitSeatUserEndTime = System.currentTimeMillis();
        String sitSeatUserUseTime = String.format(".上麦用户查询用时%d", sitSeatUserEndTime - sitSeatUserCurTime);

        // 消费用户
        long paidUserCurTime = System.currentTimeMillis();
        List<StatRtcRoomDTO> paidUser = statRoomFlowDAO.selectRoomPaidByTime(yestDayDate);
        long paidUserEndTime = System.currentTimeMillis();
        String paidUserUseTime = String.format(".消费用户查询用时%d", paidUserEndTime - paidUserCurTime);

        // 房间流水
        long roomFlowCurTime = System.currentTimeMillis();
        List<StatRtcRoomDTO> roomFlow = statRtcRoomDAO.selectFlow(yestDayDate);
        long roomFlowEndTime = System.currentTimeMillis();
        String roomFlowUseTime = String.format(".房间流水查询用时%d", roomFlowEndTime - roomFlowCurTime);

        // 直刷-普通流水
        long roomDirectNormalFlowCurTime = System.currentTimeMillis();
        List<StatRtcRoomDTO> roomDirectNormalFlow = statRoomFlowDAO.selectDirectNormalFlow(yestDayDate, BoolIntVal.False, BoolIntVal.False);
        long roomDirectNormalFlowEndTime = System.currentTimeMillis();
        String roomDirectNormalFlowUseTime = String.format(".直刷-普通流水查询用时%d", roomDirectNormalFlowEndTime - roomDirectNormalFlowCurTime);

        // 直刷-幸运流水
        long roomDirectLuckFlowCurTime = System.currentTimeMillis();
        List<StatRtcRoomDTO> roomDirectLuckFlow = statRoomFlowDAO.selectDirectNormalFlow(yestDayDate, BoolIntVal.False, BoolIntVal.True);
        long roomDirectLuckFlowEndTime = System.currentTimeMillis();
        String roomDirectLuckFlowUseTime = String.format(".直刷-幸运流水查询用时%d", roomDirectLuckFlowEndTime - roomDirectLuckFlowCurTime);

        // 背包流水
        long roomBagFlowCurTime = System.currentTimeMillis();
        List<StatRtcRoomDTO> roomBagFlow = statRoomFlowDAO.selectDirectNormalFlow(yestDayDate, BoolIntVal.True, null);
        long roomBagFlowEndTime = System.currentTimeMillis();
        String roomBagFlowUseTime = String.format(".背包流水查询用时%d", roomBagFlowEndTime - roomBagFlowCurTime);


        // 最高同时在线
        // 用户停留时长
        List<RtcRoomJoinHisDO> roomHisList = rtcRoomJoinHisDAO.selectRoomUserJoinHisByTime(yestDayStartTime, yestDayEndTime);

        // 获取所有roomId
        Set<Long> roomIds = roomHisList.stream().filter(Objects::nonNull).map(RtcRoomJoinHisDO::getRoomId).collect(Collectors.toSet());
        Map<Long, List<UserOnlineDTO>> roomUserOnlineMap = new HashMap<>();
        // 最高同时在线
        Map<Long, Integer> roomUserOnlineStatMap = new HashMap<>();
        // 用户停留时长
        Map<Long, Long> roomUserDurationStatMap = new HashMap<>();

        roomIds.forEach(roomId -> {
            roomUserOnlineMap.put(roomId, new ArrayList<>());
            roomUserOnlineStatMap.put(roomId, 0);
            roomUserDurationStatMap.put(roomId, 0L);
        });

        roomIds.forEach(roomId -> {
            List<UserOnlineDTO> onlineDTOList = roomUserOnlineMap.get(roomId);

            List<RtcRoomJoinHisDO> hisListByRoom = roomHisList.stream().filter(Objects::nonNull).filter(e -> e.getRoomId().equals(roomId)).collect(Collectors.toList());

            // 按时间倒序
            CollectionUtil.sort(hisListByRoom, (a, b) -> (int) (a.getHisCreateTime() - b.getHisCreateTime()));


            hisListByRoom.forEach(i -> {
                boolean isRecordExist = onlineDTOList.stream().anyMatch(e -> e.getUserId().equals(i.getUserId()));
                UserOnlineDTO oldOnlineDTO = onlineDTOList.stream().filter(e -> e.getUserId().equals(i.getUserId())).findFirst().orElse(new UserOnlineDTO());
                int index = onlineDTOList.indexOf(oldOnlineDTO);
                if (isRecordExist) {

                    // 如果是加入则更新加入时间
                    // 如果退出则要加在线时间
                    if (i.getJoinType() == BoolIntVal.True) {
                        oldOnlineDTO.setTimestamp(i.getHisCreateTime());
                        oldOnlineDTO.setIsInRoom(BoolIntVal.True);
                    } else {
                        oldOnlineDTO.setDuration(oldOnlineDTO.getDuration() + Math.abs(i.getHisCreateTime() - oldOnlineDTO.getTimestamp()));
                        oldOnlineDTO.setIsInRoom(BoolIntVal.False);
                    }

                    onlineDTOList.set(index, oldOnlineDTO);
                } else {

                    UserOnlineDTO newOnlineDTO = new UserOnlineDTO();

                    if (i.getJoinType() == BoolIntVal.True) {
                        newOnlineDTO.setIsInRoom(BoolIntVal.True);
                        newOnlineDTO.setTimestamp(i.getHisCreateTime());
                        newOnlineDTO.setUserId(i.getUserId());
                        newOnlineDTO.setDuration(0L);
                    } else {
                        newOnlineDTO.setIsInRoom(BoolIntVal.False);
                        newOnlineDTO.setTimestamp(i.getHisCreateTime());
                        newOnlineDTO.setUserId(i.getUserId());
                        newOnlineDTO.setDuration(0L);
                    }

                    onlineDTOList.add(newOnlineDTO);
                }

                // 判断是否为最高人数
                List<UserOnlineDTO> userOnlineCountList = onlineDTOList.stream().filter(v -> v.getIsInRoom().equals(BoolIntVal.True)).collect(Collectors.toList());
                if (roomUserOnlineStatMap.get(roomId) < userOnlineCountList.size()) {
                    roomUserOnlineStatMap.put(roomId, userOnlineCountList.size());
                }
            });

            // 总在线时长
            List<UserOnlineDTO> newOnlineDTOList = roomUserOnlineMap.get(roomId);
            Long duration = newOnlineDTOList.stream().map(UserOnlineDTO::getDuration).reduce(0L, Long::sum);
            roomUserDurationStatMap.put(roomId, duration);
        });

        //获得房间类型
        Map<Long, RtcRoomDO> roomBusinessMap = rtcRoomDAO.selectRoomBusinessMap(new ArrayList<>(roomIds));
        log.info("房间消费统计：：roomBusinessMap：" + JSON.toJSONString(roomBusinessMap));

        List<StatRtcRoomGradeDO> list = new ArrayList<>();

        // 房间活跃用户数
        activityUser.forEach(val -> {

            StatRtcRoomGradeDO oldDO = list.stream().filter(i -> i.getRoomId().equals(val.getRoomId())).findFirst().orElse(null);
            if (oldDO != null) {
                int index = list.indexOf(oldDO);
                oldDO.setActivityUserCount(val.getCount());
                list.set(index, oldDO);
            } else {
                StatRtcRoomGradeDO aDo = new StatRtcRoomGradeDO();
                aDo.setRoomId(val.getRoomId());
                aDo.setHisId(identityService.getId());
                aDo.setCreateDate(yestDayDate);
                aDo.setCreateTime(System.currentTimeMillis());

                aDo.setActivityUserCount(val.getCount());
                list.add(aDo);
            }
        });

        // 收藏用户
        favoriteUser.forEach(val -> {

            StatRtcRoomGradeDO oldDO = list.stream().filter(i -> i.getRoomId().equals(val.getRoomId())).findFirst().orElse(null);
            if (oldDO != null) {
                int index = list.indexOf(oldDO);
                oldDO.setFavoriteUserCount(val.getCount());
                list.set(index, oldDO);
            } else {
                StatRtcRoomGradeDO aDo = new StatRtcRoomGradeDO();
                aDo.setRoomId(val.getRoomId());
                aDo.setHisId(identityService.getId());
                aDo.setCreateDate(yestDayDate);
                aDo.setCreateTime(System.currentTimeMillis());

                aDo.setFavoriteUserCount(val.getCount());
                list.add(aDo);
            }
        });

        // 上麦用户
        sitSeatUser.forEach(val -> {

            StatRtcRoomGradeDO oldDO = list.stream().filter(i -> i.getRoomId().equals(val.getRoomId())).findFirst().orElse(null);
            if (oldDO != null) {
                int index = list.indexOf(oldDO);
                oldDO.setSitSeatUserCount(val.getCount());
                list.set(index, oldDO);
            } else {
                StatRtcRoomGradeDO aDo = new StatRtcRoomGradeDO();
                aDo.setRoomId(val.getRoomId());
                aDo.setHisId(identityService.getId());
                aDo.setCreateDate(yestDayDate);
                aDo.setCreateTime(System.currentTimeMillis());

                aDo.setSitSeatUserCount(val.getCount());
                list.add(aDo);
            }
        });

        // 消费用户
        paidUser.forEach(val -> {

            StatRtcRoomGradeDO oldDO = list.stream().filter(i -> i.getRoomId().equals(val.getRoomId())).findFirst().orElse(null);
            if (oldDO != null) {
                int index = list.indexOf(oldDO);
                oldDO.setPaidUserCount(val.getCount());
                list.set(index, oldDO);
            } else {
                StatRtcRoomGradeDO aDo = new StatRtcRoomGradeDO();
                aDo.setRoomId(val.getRoomId());
                aDo.setHisId(identityService.getId());
                aDo.setCreateDate(yestDayDate);
                aDo.setCreateTime(System.currentTimeMillis());

                aDo.setPaidUserCount(val.getCount());
                list.add(aDo);
            }
        });

        // 房间流水
        roomFlow.forEach(val -> {

            StatRtcRoomGradeDO oldDO = list.stream().filter(i -> i.getRoomId().equals(val.getRoomId())).findFirst().orElse(null);
            if (oldDO != null) {
                int index = list.indexOf(oldDO);
                oldDO.setRoomFlow(val.getCount());
                list.set(index, oldDO);
            } else {
                StatRtcRoomGradeDO aDo = new StatRtcRoomGradeDO();
                aDo.setRoomId(val.getRoomId());
                aDo.setHisId(identityService.getId());
                aDo.setCreateDate(yestDayDate);
                aDo.setCreateTime(System.currentTimeMillis());

                aDo.setRoomFlow(val.getCount());
                list.add(aDo);
            }
        });

        // 直刷-普通流水
        roomDirectNormalFlow.forEach(val -> {

            StatRtcRoomGradeDO oldDO = list.stream().filter(i -> i.getRoomId().equals(val.getRoomId())).findFirst().orElse(null);
            if (oldDO != null) {
                int index = list.indexOf(oldDO);
                oldDO.setDirectNormalFlow(val.getCount());
                list.set(index, oldDO);
            } else {
                StatRtcRoomGradeDO aDo = new StatRtcRoomGradeDO();
                aDo.setRoomId(val.getRoomId());
                aDo.setHisId(identityService.getId());
                aDo.setCreateDate(yestDayDate);
                aDo.setCreateTime(System.currentTimeMillis());

                aDo.setDirectNormalFlow(val.getCount());
                list.add(aDo);
            }
        });

        // 直刷-幸运流水
        roomDirectLuckFlow.forEach(val -> {

            StatRtcRoomGradeDO oldDO = list.stream().filter(i -> i.getRoomId().equals(val.getRoomId())).findFirst().orElse(null);
            if (oldDO != null) {
                int index = list.indexOf(oldDO);
                oldDO.setDirectLuckFlow(val.getCount());
                list.set(index, oldDO);
            } else {
                StatRtcRoomGradeDO aDo = new StatRtcRoomGradeDO();
                aDo.setRoomId(val.getRoomId());
                aDo.setHisId(identityService.getId());
                aDo.setCreateDate(yestDayDate);
                aDo.setCreateTime(System.currentTimeMillis());

                aDo.setDirectLuckFlow(val.getCount());
                list.add(aDo);
            }
        });

        // 直刷-幸运流水
        roomBagFlow.forEach(val -> {

            StatRtcRoomGradeDO oldDO = list.stream().filter(i -> i.getRoomId().equals(val.getRoomId())).findFirst().orElse(null);
            if (oldDO != null) {
                int index = list.indexOf(oldDO);
                oldDO.setBagFlow(val.getCount());
                list.set(index, oldDO);
            } else {
                StatRtcRoomGradeDO aDo = new StatRtcRoomGradeDO();
                aDo.setRoomId(val.getRoomId());
                aDo.setHisId(identityService.getId());
                aDo.setCreateDate(yestDayDate);
                aDo.setCreateTime(System.currentTimeMillis());

                aDo.setBagFlow(val.getCount());
                list.add(aDo);
            }
        });

        // 背包消费档次记录
        Map<Long, List<GradeVO>> gradeBagVOMap = new HashMap<>();
        gradeVOS.stream().filter(Objects::nonNull).forEach(val -> {

            if (!roomIds.isEmpty()) {
                List<StatRtcRoomDTO> gradeList = statRoomFlowDAO.selectListByRoomIds(roomIds, Optional.ofNullable(val.getStart()).orElse(0) * 100, Optional.ofNullable(val.getEnd()).orElse(0) * 100, yestDayDate, yestDayDate, BoolIntVal.True);
                Map<Long, StatRtcRoomDTO> gradeMap = gradeList.stream().collect(Collectors.toMap(StatRtcRoomDTO::getRoomId, Function.identity()));

                gradeMap.forEach((key, value) -> {

                    List<GradeVO> gradeVORoom = new ArrayList<>();

                    if (gradeBagVOMap.containsKey(key)) {
                        gradeVORoom = gradeBagVOMap.get(key);
                    }

                    GradeVO g = new GradeVO();
                    BeanUtils.copyProperties(val, g);
                    g.setValue(value.getCount().intValue());
                    gradeVORoom.add(g);
                    gradeBagVOMap.put(key, gradeVORoom);
                });
            }
        });

        roomIds.forEach(roomId -> {

            StatRtcRoomGradeDO oldDO = list.stream().filter(i -> i.getRoomId().equals(roomId)).findFirst().orElse(null);
            if (oldDO != null) {
                int index = list.indexOf(oldDO);
                oldDO.setBagFlowGrade(JSON.toJSONString(gradeBagVOMap.get(roomId)));
                list.set(index, oldDO);
            } else {
                StatRtcRoomGradeDO aDo = new StatRtcRoomGradeDO();
                aDo.setRoomId(roomId);
                aDo.setHisId(identityService.getId());
                aDo.setCreateDate(yestDayDate);
                aDo.setCreateTime(System.currentTimeMillis());
                aDo.setBagFlowGrade(JSON.toJSONString(gradeBagVOMap.get(roomId)));
                list.add(aDo);
            }
        });

        // 直刷消费档次记录
        Map<Long, List<GradeVO>> gradeVOMap = new HashMap<>();
        gradeVOS.stream().filter(Objects::nonNull).forEach(val -> {

            if (!roomIds.isEmpty()) {
                List<StatRtcRoomDTO> gradeList = statRoomFlowDAO.selectListByRoomIds(roomIds, Optional.ofNullable(val.getStart()).orElse(0) * 100, Optional.ofNullable(val.getEnd()).orElse(0) * 100, yestDayDate, yestDayDate, BoolIntVal.False);

                Map<Long, StatRtcRoomDTO> gradeMap = gradeList.stream().collect(Collectors.toMap(StatRtcRoomDTO::getRoomId, Function.identity()));

                gradeMap.forEach((key, value) -> {

                    List<GradeVO> gradeVORoom = new ArrayList<>();

                    if (gradeVOMap.containsKey(key)) {
                        gradeVORoom = gradeVOMap.get(key);
                    }

                    GradeVO g = new GradeVO();
                    BeanUtils.copyProperties(val, g);
                    g.setValue(value.getCount().intValue());
                    gradeVORoom.add(g);
                    gradeVOMap.put(key, gradeVORoom);
                });
            }
        });

        roomIds.forEach(roomId -> {

            StatRtcRoomGradeDO oldDO = list.stream().filter(i -> i.getRoomId().equals(roomId)).findFirst().orElse(null);
            if (oldDO != null) {
                int index = list.indexOf(oldDO);
                oldDO.setDirectFlowGrade(JSON.toJSONString(gradeVOMap.get(roomId)));
                list.set(index, oldDO);
            } else {
                StatRtcRoomGradeDO aDo = new StatRtcRoomGradeDO();
                aDo.setRoomId(roomId);
                aDo.setHisId(identityService.getId());
                aDo.setCreateDate(yestDayDate);
                aDo.setCreateTime(System.currentTimeMillis());
                aDo.setDirectFlowGrade(JSON.toJSONString(gradeVOMap.get(roomId)));

                list.add(aDo);
            }
        });


        // 最高同时在线
        for (Map.Entry<Long, Integer> entry : roomUserOnlineStatMap.entrySet()) {
            StatRtcRoomGradeDO oldDO = list.stream().filter(i -> i.getRoomId().equals(entry.getKey())).findFirst().orElse(null);
            if (oldDO != null) {
                int index = list.indexOf(oldDO);
                oldDO.setHighestOnlineUserCount(entry.getValue().longValue());
                list.set(index, oldDO);
            } else {
                StatRtcRoomGradeDO aDo = new StatRtcRoomGradeDO();
                aDo.setRoomId(entry.getKey());
                aDo.setHisId(identityService.getId());
                aDo.setCreateDate(yestDayDate);
                aDo.setCreateTime(System.currentTimeMillis());

                aDo.setHighestOnlineUserCount(entry.getValue().longValue());
                list.add(aDo);
            }
        }

        // 用户停留时长
        for (Map.Entry<Long, Long> entry : roomUserDurationStatMap.entrySet()) {

            StatRtcRoomGradeDO oldDO = list.stream().filter(i -> i.getRoomId().equals(entry.getKey())).findFirst().orElse(null);
            if (oldDO != null) {
                int index = list.indexOf(oldDO);
                oldDO.setUserStayTimes(entry.getValue());
                list.set(index, oldDO);
            } else {
                StatRtcRoomGradeDO aDo = new StatRtcRoomGradeDO();
                aDo.setRoomId(entry.getKey());
                aDo.setHisId(identityService.getId());
                aDo.setCreateDate(yestDayDate);
                aDo.setCreateTime(System.currentTimeMillis());

                aDo.setUserStayTimes(entry.getValue());
                list.add(aDo);
            }
        }

        ////附加值 融云弹幕相关
        Map<Long, StatBarrageRsp> statBarrageRspMap = getStatBarrage(yestDayStartTime, yestDayEndTime, new ArrayList<Long>(roomIds));

        //弹幕与发言人数
        for (StatRtcRoomGradeDO gradeDO : list) {
            gradeDO.setDimension(0);

            RtcRoomDO roomDO = roomBusinessMap.get(gradeDO.getRoomId());
            if (roomDO != null) {
                gradeDO.setBusinessType(roomDO.getBusinessType());
            }

            StatBarrageRsp barrageRsp = statBarrageRspMap.get(gradeDO.getRoomId());
            if (barrageRsp != null) {
                gradeDO.setBarrageNum(barrageRsp.getBarrage().longValue());
                gradeDO.setSpeakNum(barrageRsp.getPeople().longValue());
            }
        }

        try {
            list.removeIf(val -> val.getRoomId() == 0L);
//            int result = statRtcRoomGradeDAO.insertBatch(list);
            if (list.size() > 0) {
                //开始评分
                calculateRoomScore(list);
                int result = statRtcRoomGradeDAO.batchInsertOrUpdate(list);
                log.info("插入{}条数据成功", result);
            }
        } catch (Exception e) {
            log.error("生成房间消费统计执行出错", e);
            return ReturnT.FAIL;
        }
        long endTime = System.currentTimeMillis();
        log.info("执行成功,共耗时{}s", (endTime - curTime) / 1000);

        return new ReturnT<>(yestDayDate + "数据." + activityUserUseTime + favoriteUserUseTime + sitSeatUserUseTime + paidUserUseTime + roomFlowUseTime + roomDirectNormalFlowUseTime + roomDirectLuckFlowUseTime + roomBagFlowUseTime + ".共耗时:" + (endTime - curTime));
    }

    /**
     * 每天从redis中持久化数据
     *
     * @param param
     * @return
     */
    @XxlJob(value = JobHandlers.STAT_PLATFORM_ONLINE_TASK)
    @Transactional(rollbackFor = Exception.class)
    public ReturnT<String> statPlatformOnlineHandler(String param) throws ParseException {

        log.info("平台在线时长统计: {}", param);

        // 昨天日期
        SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMdd");
        Date yestDay = param != null && !"".equals(param) ? formatter.parse(param) : DateUtil.getYesterday(new Date());

        // 昨天时间YYYYMMDD
        int yestDayDate = DateUtil.getIntDate(yestDay);
        int nowDate = DateUtil.getIntDate();
        long curTime = System.currentTimeMillis();

        String onlineCountMax = redisService.get(RedisUserKeys.getPlatformOnlineUserMax(yestDayDate));
        log.info("平台最高在线人数: {}", onlineCountMax);

        Map<Object, Object> map = redisService.hashGetAll(RedisUserKeys.getPlatformOnlineUserDuration(yestDayDate));

        if (isDev()) {
            log.info("最高在线时长: {}", JSON.toJSONString(map));
        }

        Long duration = 0L;

        for (Map.Entry<Object, Object> entry : map.entrySet()) {
            duration += Convert.toLong(entry.getValue());
        }

        // 直刷消费档次
        List<GradeVO> grade = new ArrayList<>();
        gradeVOS.stream().filter(Objects::nonNull).forEach(val -> {
            List<StatRtcRoomDTO> listMap = statRoomFlowDAO.selectListByRoomIds(new ArrayList<Long>(Arrays.asList(0L)), Optional.ofNullable(val.getStart()).orElse(0) * 100, Optional.ofNullable(val.getEnd()).orElse(0) * 100, yestDayDate, yestDayDate, BoolIntVal.False);
            GradeVO gradeVO = new GradeVO();
            gradeVO.setStart(val.getStart());
            gradeVO.setEnd(val.getEnd());

            if (listMap != null && listMap.size() > 0 && listMap.get(0) != null) {
                gradeVO.setValue(listMap.get(0).getCount().intValue());
            }
            grade.add(gradeVO);
        });

        // 背包消费档次
        List<GradeVO> gradeBag = new ArrayList<>();
        gradeVOS.stream().filter(Objects::nonNull).forEach(val -> {
            List<StatRtcRoomDTO> listMap = statRoomFlowDAO.selectListByRoomIds(new ArrayList<Long>(Arrays.asList(0L)), Optional.ofNullable(val.getStart()).orElse(0) * 100, Optional.ofNullable(val.getEnd()).orElse(0) * 100, yestDayDate, yestDayDate, BoolIntVal.True);
            GradeVO gradeVO = new GradeVO();
            gradeVO.setStart(val.getStart());
            gradeVO.setEnd(val.getEnd());

            if (listMap != null && listMap.size() > 0 && listMap.get(0) != null) {
                gradeVO.setValue(listMap.get(0).getCount().intValue());
            }
            gradeBag.add(gradeVO);
        });

        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("roomId", 0L);
        queryWrapper.eq("createDate", yestDayDate);
        if (statRtcRoomGradeDAO.selectCount(queryWrapper) > 0) {
            StatRtcRoomGradeDO gradeDO = new StatRtcRoomGradeDO();
            gradeDO.setUserStayTimes(duration);
            gradeDO.setHighestOnlineUserCount(onlineCountMax == null ? 0L : Long.parseLong(onlineCountMax));
            gradeDO.setCreateTime(System.currentTimeMillis());
            gradeDO.setCreateDate(yestDayDate);
            gradeDO.setDirectFlowGrade(JSON.toJSONString(grade));
            gradeDO.setBagFlowGrade(JSON.toJSONString(gradeBag));
            gradeDO.setRoomId(0L);
            UpdateWrapper uw = new UpdateWrapper();
            uw.setPo(gradeDO);
            uw.setQw(queryWrapper);
            int result = statRtcRoomGradeDAO.update(uw);
            log.info("更新{}条数据成功", result);
        } else {
            StatRtcRoomGradeDO gradeDO = new StatRtcRoomGradeDO();
            gradeDO.setUserStayTimes(duration);
            gradeDO.setHighestOnlineUserCount(onlineCountMax == null ? 0L : Long.parseLong(onlineCountMax));
            gradeDO.setCreateTime(System.currentTimeMillis());
            gradeDO.setCreateDate(yestDayDate);
            gradeDO.setDirectFlowGrade(JSON.toJSONString(grade));
            gradeDO.setBagFlowGrade(JSON.toJSONString(gradeBag));
            gradeDO.setRoomId(0L);
            gradeDO.setHisId(identityService.getId());
            int result = statRtcRoomGradeDAO.insert(gradeDO);
            log.info("插入{}条数据成功", result);
        }

        // 将昨日的在线用户放到今日数据里
        Map<Object, Object> data = redisService.hashGetAll(RedisUserKeys.getPlatformOnlineUserDuration(yestDayDate));
        data.forEach((key, value) -> {
            redisService.hashPut(RedisUserKeys.getPlatformUserLastOnline(nowDate), String.valueOf(key), String.valueOf(System.currentTimeMillis()));
        });

        redisService.delKey(RedisUserKeys.getPlatformOnlineUserMax(yestDayDate));
        redisService.delKey(RedisUserKeys.getPlatformOnlineUserDuration(yestDayDate));

        log.info("删除key成功:{},{}", RedisUserKeys.getPlatformOnlineUserMax(yestDayDate), RedisUserKeys.getPlatformOnlineUserDuration(yestDayDate));

        long endTime = System.currentTimeMillis();
        log.info("执行成功,共耗时{}s", (endTime - curTime) / 1000);

        return new ReturnT<>(yestDayDate + ".共耗时:" + (endTime - curTime));
    }

    /**
     * 新增统计表字段后，需要对房间类型字段进行数据维护(只需要初始执行一次，后续状态统计消费相关会写入类型)
     *
     * @param param
     * @return
     */
    @XxlJob(value = JobHandlers.STAT_RTC_ROOM_GRADE_BUSINESSTYPE_TASK)
    @Transactional(rollbackFor = Exception.class)
    public ReturnT<String> statGradeRtcRoomTypeHandle(String param) {
        int pageSize = 2000;
        int pageCursor = 1;
        boolean isRunloop = true;
        while (isRunloop) {
            //批量获得房间ID
            List<Long> roomIds = statRtcRoomGradeDAO.getStatRoomIDList((pageCursor - 1) * pageSize, pageSize);
            if (CollectionUtils.isEmpty(roomIds)) {
                break;
            }
            if (roomIds.size() < pageSize) {
                isRunloop = false;
            }

            //获得房间类型
            Map<Long, RtcRoomDO> roomBusinessMap = rtcRoomDAO.selectRoomBusinessMap(new ArrayList<>(roomIds));
            List<RtcRoomDO> roomDOList = new ArrayList<>(roomIds.size());
            roomIds.forEach(k -> {
                if (roomBusinessMap.containsKey(k)) {
                    RtcRoomDO roomDO = new RtcRoomDO();
                    roomDO.setRoomId(k);
                    roomDO.setBusinessType(roomBusinessMap.get(k).getBusinessType());
                    roomDOList.add(roomDO);
                }
            });
            statRtcRoomGradeDAO.batchUpdateRoomType(roomDOList);

            pageCursor++;
        }

        return ReturnT.SUCCESS;
    }

    /**
     * 周统计
     *
     * @param param
     * @return
     */
    @XxlJob(value = JobHandlers.STAT_RTC_ROOM_GRADE_WEEK_TASK)
    @Transactional(rollbackFor = Exception.class)
    public ReturnT<String> statWeekRtcRoomGradeHandler(String param) throws ParseException {
        long weekStartTime = DateUtil.getLastWeekStartTimeStamp();
        long weekEndTime = DateUtil.getLastWeekEndTimeStamp();

        Integer weekStartDate = DateUtil.getIntDate(weekStartTime);
        Integer weekEndDate = DateUtil.getIntDate(weekEndTime);

        log.info("房间消费统计：weekStartDate：" + weekStartDate);
        log.info("房间消费统计：weekEndDate：" + weekEndDate);

        List<StatRtcRoomGradeDO> rtcRoomGradeDOList = new ArrayList<StatRtcRoomGradeDO>();

        QueryWrapper qu = new QueryWrapper();
        qu.ge("createDate", weekStartDate);
        qu.le("createDate", weekEndDate);
        qu.eq("dimension", 0);
        qu.ne("roomId", 0);

        int currentPage = 1;
        int pageSize = 10000;
        boolean isRunloop = true;
        while (isRunloop) {
            qu.setPageCursor((currentPage - 1) * pageSize);
            qu.setPageSize(pageSize);
            List<StatRtcRoomGradeDO> searchList = statRtcRoomGradeDAO.selectPageList(qu);
            if (CollectionUtils.isEmpty(searchList) || searchList.size() == 0) {
                break;
            }
            if (searchList.size() < pageSize) {
                isRunloop = false;
            }
            rtcRoomGradeDOList.addAll(searchList);
            currentPage++;
        }

        //获得统计数据列表
        List<StatRtcRoomGradeDO> statRtcRoomGradeDOList = statDimensionList(rtcRoomGradeDOList, 1, weekStartDate);
        //计算房间评分
        calculateRoomScore(statRtcRoomGradeDOList);
        if (rtcRoomGradeDOList.size() > 0) {
            statRtcRoomGradeDAO.batchInsertOrUpdate(statRtcRoomGradeDOList);
        }

        log.info("房间消费统计：执行完成【周】维度 " + statRtcRoomGradeDOList.size() + "条");

        return ReturnT.SUCCESS;
    }

    /**
     * 月统计
     *
     * @param param
     * @return
     */
    @XxlJob(value = JobHandlers.STAT_RTC_ROOM_GRADE_MONTH_TASK)
    @Transactional(rollbackFor = Exception.class)
    public ReturnT<String> statMonthRtcRoomGradeHandler(String param) {
        Date gatherMonthDate = new Date();

        long monthStartTime = DateUtil.getMonthStart(gatherMonthDate);
        long monthEndTime = DateUtil.getMonthEnd(gatherMonthDate);

        Integer monthStartDate = DateUtil.getIntDate(monthStartTime);
        Integer monthEndDate = DateUtil.getIntDate(monthEndTime);

        List<StatRtcRoomGradeDO> rtcRoomGradeDOList = new ArrayList<StatRtcRoomGradeDO>();

        QueryWrapper qu = new QueryWrapper();
        qu.ge("createDate", monthStartDate);
        qu.le("createDate", monthEndDate);
        qu.eq("dimension", 0);
        qu.ne("roomId", 0);

        int currentPage = 1;
        int pageSize = 10000;
        boolean isRunloop = true;
        while (isRunloop) {
            qu.setPageCursor((currentPage - 1) * pageSize);
            qu.setPageSize(pageSize);
            List<StatRtcRoomGradeDO> searchList = statRtcRoomGradeDAO.selectPageList(qu);
            if (CollectionUtils.isEmpty(searchList) || searchList.size() == 0) {
                break;
            }
            if (searchList.size() < pageSize) {
                isRunloop = false;
            }
            rtcRoomGradeDOList.addAll(searchList);
            currentPage++;
        }

        //获得统计数据列表
        List<StatRtcRoomGradeDO> statRtcRoomGradeDOList = statDimensionList(rtcRoomGradeDOList, 2, monthStartDate);
        calculateRoomScore(statRtcRoomGradeDOList);

        if (statRtcRoomGradeDOList.size() > 0) {
            statRtcRoomGradeDAO.batchInsertOrUpdate(statRtcRoomGradeDOList);
        }

        log.info("房间消费统计：执行完成【月】维度 " + statRtcRoomGradeDOList.size() + "条");

        return ReturnT.SUCCESS;
    }

    /**
     * 查询统计的数据
     *
     * @param gradeDOList     查询出来的维度数据，
     * @param targetDimension 目标维度类型 1:周 2：月
     * @param createDate      目标维度起始时间 yyyyMMdd
     * @return
     */
    private List<StatRtcRoomGradeDO> statDimensionList(List<StatRtcRoomGradeDO> gradeDOList, int targetDimension, Integer createDate) {
        long currentTime = System.currentTimeMillis();
        //目标统计数据
        Map<Long, StatRtcRoomGradeDO> targetGradeMap = new HashMap<>();
        gradeDOList.forEach(k -> {
            if (targetGradeMap.containsKey(k.getRoomId())) {
                StatRtcRoomGradeDO gradeDO = targetGradeMap.get(k.getRoomId());
                //活跃用户
                long targetActivityCount = mergeCount(gradeDO.getActivityUserCount(), k.getActivityUserCount());
                gradeDO.setActivityUserCount(targetActivityCount);
                //收藏用户
                long targetFavoriteCount = mergeCount(gradeDO.getFavoriteUserCount(), k.getFavoriteUserCount());
                gradeDO.setFavoriteUserCount(targetFavoriteCount);
                //上麦用户
                long targetSitSeatCount = mergeCount(gradeDO.getSitSeatUserCount(), k.getSitSeatUserCount());
                gradeDO.setSitSeatUserCount(targetSitSeatCount);
                //消费用户
                long targetPaidCount = mergeCount(gradeDO.getPaidUserCount(), k.getPaidUserCount());
                gradeDO.setPaidUserCount(targetPaidCount);
                //弹幕用户量
                long targetSpeakNumCount = mergeCount(gradeDO.getSpeakNum(), k.getSpeakNum());
                gradeDO.setSpeakNum(targetSpeakNumCount);
                //弹幕数量
                long targetBarrgeNumCount = mergeCount(gradeDO.getBarrageNum(), k.getBarrageNum());
                gradeDO.setBarrageNum(targetBarrgeNumCount);
                //房间钻石流水
                long targetRoomFlowCount = mergeCount(gradeDO.getRoomFlow(), k.getRoomFlow());
                gradeDO.setRoomFlow(targetRoomFlowCount);
                //最高同时在线
                long targetHighestOnlineCount = Optional.ofNullable(gradeDO.getHighestOnlineUserCount()).orElse(0L);
                long kindHighestOnlineCount = Optional.ofNullable(k.getHighestOnlineUserCount()).orElse(0L);
                if (kindHighestOnlineCount > targetHighestOnlineCount) {
                    gradeDO.setHighestOnlineUserCount(kindHighestOnlineCount);
                }
                //直刷普通流水
                long targetDirectNormalFlowCount = mergeCount(gradeDO.getDirectNormalFlow(), k.getDirectNormalFlow());
                gradeDO.setDirectNormalFlow(targetDirectNormalFlowCount);
                //直刷幸运流水
                long targetDirectLuckFlowCount = mergeCount(gradeDO.getDirectLuckFlow(), k.getDirectLuckFlow());
                gradeDO.setDirectLuckFlow(targetDirectLuckFlowCount);
                //直刷背包流水
                long targetBagFlowCount = mergeCount(gradeDO.getBagFlow(), k.getBagFlow());
                gradeDO.setBagFlow(targetBagFlowCount);

                //直刷消费档次统计
                String directFlowGrade = gradeDO.getDirectFlowGrade();
                String kindDirectFlowGrade = k.getDirectFlowGrade();

                //目标集合不存在值
                boolean isNotHaveDirectFlowGrade = StringUtils.isBlank(directFlowGrade) || "null".equals(directFlowGrade);
                //当前操作存在值
                boolean isHaveKindDirectFlowGrade = StringUtils.isNotBlank(kindDirectFlowGrade) || "null".equals(kindDirectFlowGrade) == false;
                if (isNotHaveDirectFlowGrade && isHaveKindDirectFlowGrade) {
                    gradeDO.setDirectFlowGrade(k.getDirectFlowGrade());
                } else {
                    if (isHaveKindDirectFlowGrade) {
                        List<GradeVO> directFlowGradeList = new ArrayList<>();
                        List<GradeVO> kindDirectFlowGradeList = JSON.parseArray(kindDirectFlowGrade, GradeVO.class);
                        List<GradeVO> targetFlowGradeList = JSON.parseArray(directFlowGrade, GradeVO.class);

                        if (CollectionUtils.isNotEmpty(kindDirectFlowGradeList) && kindDirectFlowGradeList.size() > 0) {
                            directFlowGradeList.addAll(kindDirectFlowGradeList);
                        }
                        if (CollectionUtils.isNotEmpty(targetFlowGradeList) && targetFlowGradeList.size() > 0) {
                            directFlowGradeList.addAll(targetFlowGradeList);
                        }

                        List<GradeVO> directGradeVOList = defaultGradeVOList();
                        directFlowGradeList.forEach(item -> {
                            if (directGradeVOList.stream().anyMatch(m -> m.getStart().equals(item.getStart()))) {
                                directGradeVOList.forEach(flowGrade -> {
                                    if (flowGrade.getStart().equals(item.getStart())) {
                                        Integer gradeValue = Optional.ofNullable(flowGrade.getValue()).orElse(0);
                                        Integer itemGradeValue = Optional.ofNullable(item.getValue()).orElse(0);
                                        flowGrade.setValue(gradeValue + itemGradeValue);
                                    }
                                });
                            }
                        });
                        gradeDO.setDirectFlowGrade(JSON.toJSONString(directGradeVOList));
                    }
                }

                //直刷消费档次统计
                String bagFlowGrade = gradeDO.getDirectFlowGrade();
                if (StringUtils.isBlank(bagFlowGrade) || "null".equals(bagFlowGrade)) {
                    gradeDO.setBagFlowGrade(k.getBagFlowGrade());
                } else {
                    List<GradeVO> bagGradeVOList = defaultGradeVOList();
                    List<GradeVO> bagFlowGradeList = JSON.parseArray(bagFlowGrade, GradeVO.class);
                    bagFlowGradeList.forEach(item -> {
                        if (bagGradeVOList.stream().anyMatch(m -> m.getStart().equals(item.getStart()))) {
                            bagGradeVOList.forEach(flowGrade -> {
                                if (flowGrade.getStart().equals(item.getStart())) {
                                    Integer gradeValue = Optional.ofNullable(flowGrade.getValue()).orElse(0);
                                    Integer itemGradeValue = Optional.ofNullable(item.getValue()).orElse(0);
                                    flowGrade.setValue(gradeValue + itemGradeValue);
                                }
                            });
                        }
                    });
                    gradeDO.setBagFlowGrade(JSON.toJSONString(bagGradeVOList));
                }
            } else {
                k.setHisId(identityService.getId());
                k.setCreateDate(createDate);
                k.setCreateTime(currentTime);
                k.setDimension(targetDimension);
                targetGradeMap.put(k.getRoomId(), k);
            }
        });

        return new ArrayList<StatRtcRoomGradeDO>(targetGradeMap.values());
    }

    private long mergeCount(Long targetNum, Long kindNum) {
        return Optional.ofNullable(targetNum).orElse(0L) + Optional.ofNullable(kindNum).orElse(0L);
    }

    /**
     * 获得统计弹幕
     *
     * @param roomIds 房间列表
     */
    protected Map<Long, StatBarrageRsp> getStatBarrage(Long startTime, Long endTime, List<Long> roomIds) {
        String url = String.format("%s/%s", apiNotifyRongcloud, "v1/rongCloud/getStatBarrage");
        Map<String, String> params = new HashMap<>();
        params.put("startTime", "" + startTime);
        params.put("endTime", "" + (endTime + 1));
        params.put("list", StringUtils.join(roomIds, ","));
        String rsp;
        if (isDev()) {
            log.info(LogUtil.devLog(String.format("请求房间弹幕统计列表 url=%s params=%s", url, JSON.toJSONString(params))));
        }
        try {
            rsp = HttpUtil.postForm(url, params);
            if (isDev()) {
                log.info("请求房间弹幕统计列表rsp={}", LogUtil.devLog(rsp));
            }
        } catch (IOException e) {
            log.error(String.format("请求房间弹幕统计列表失败 url=%s params=%s", url, JSON.toJSONString(params)), e);
            return new HashMap<>();
        }

        JSONObject rspJson = JSON.parseObject(rsp);
        // 1:success  0:error
        Integer code = rspJson.getInteger("code");
        if (code == null || code != 1) {
            // 返回结果错误
            log.error(String.format("请求房间弹幕统计列表返回结果错误code url=%s params=%s rsp=%s",
                    url, JSON.toJSONString(params), rsp));
            return new HashMap<>();
        }
        JSONArray data = rspJson.getJSONArray("data");
        if (CollectionUtils.isEmpty(data)) {
            log.error(String.format("请求房间弹幕统计列表返回结果错误data url=%s params=%s rsp=%s",
                    url, JSON.toJSONString(params), rsp));
            return new HashMap<>();
        }

        List<StatBarrageRsp> resBarrageList = JSON.parseArray(data.toJSONString(), StatBarrageRsp.class);
        return resBarrageList.stream().collect(Collectors.toMap(StatBarrageRsp::getRoomId, Function.identity()));
    }

    /**
     * 计算房间评分
     * 只会更具列表中的值进行计算（所以list参数是具有维度属性：日、周、月）
     *
     * @param list
     */
    protected void calculateRoomScore(List<StatRtcRoomGradeDO> list) {
        long activeUserTotal = list.stream().filter(k -> k.getActivityUserCount() != null).mapToLong(StatRtcRoomGradeDO::getActivityUserCount).sum();
        long favoriteUserTotal = list.stream().filter(k -> k.getFavoriteUserCount() != null).mapToLong(StatRtcRoomGradeDO::getFavoriteUserCount).sum();
        long sitSeatUserTotal = list.stream().filter(k -> k.getSitSeatUserCount() != null).mapToLong(StatRtcRoomGradeDO::getSitSeatUserCount).sum();
        long paidUserTotal = list.stream().filter(k -> k.getPaidUserCount() != null).mapToLong(StatRtcRoomGradeDO::getPaidUserCount).sum();
        long speakTotal = list.stream().filter(k -> k.getSpeakNum() != null).mapToLong(StatRtcRoomGradeDO::getSpeakNum).sum();
        long barrageTotal = list.stream().filter(k -> k.getBarrageNum() != null).mapToLong(StatRtcRoomGradeDO::getBarrageNum).sum();
        long highestOnlineUserTotal = list.stream().filter(k -> k.getHighestOnlineUserCount() != null).mapToLong(StatRtcRoomGradeDO::getHighestOnlineUserCount).sum();
        long userStayTimesTotal = list.stream().filter(k -> k.getUserStayTimes() != null).mapToLong(StatRtcRoomGradeDO::getUserStayTimes).sum();

        //key:对应数据库表中的数值字段
        Map<Long, Long> gradeMap = getDefaultGradeMap();
        addGradeMapTotal(gradeMap, 1001L, Long.valueOf(activeUserTotal));
        addGradeMapTotal(gradeMap, 1002L, Long.valueOf(favoriteUserTotal));
        addGradeMapTotal(gradeMap, 1003L, Long.valueOf(sitSeatUserTotal));
        addGradeMapTotal(gradeMap, 1004L, Long.valueOf(paidUserTotal));
        addGradeMapTotal(gradeMap, 1005L, Long.valueOf(speakTotal));
        addGradeMapTotal(gradeMap, 1006L, Long.valueOf(barrageTotal));
        addGradeMapTotal(gradeMap, 1007L, Long.valueOf(highestOnlineUserTotal));
        addGradeMapTotal(gradeMap, 1008L, Long.valueOf(userStayTimesTotal));

        for (StatRtcRoomGradeDO gradeDO : list) {
            String directFlowGrade = gradeDO.getDirectFlowGrade();
            if (StringUtils.isNotBlank(directFlowGrade) && "null".equals(directFlowGrade) == false) {
                List<GradeVO> gradeList = JSON.parseArray(directFlowGrade, GradeVO.class);
                for (GradeVO gradeVO : gradeList) {
                    int start = gradeVO.getStart();
                    int value = Optional.ofNullable(gradeVO.getValue()).orElse(0);
                    switch (start) {
                        case 0:
                            addGradeMapTotal(gradeMap, 1009L, Long.valueOf(value));
                            break;
                        case 100:
                            addGradeMapTotal(gradeMap, 1010L, Long.valueOf(value));
                            break;
                        case 500:
                            addGradeMapTotal(gradeMap, 1011L, Long.valueOf(value));
                            break;
                        case 1000:
                            addGradeMapTotal(gradeMap, 1012L, Long.valueOf(value));
                            break;
                        case 3000:
                            addGradeMapTotal(gradeMap, 1013L, Long.valueOf(value));
                            break;
                        default:
                            break;
                    }
                }
            }

            String bagFlowGrade = gradeDO.getBagFlowGrade();
            if (StringUtils.isNotBlank(bagFlowGrade) && "null".equals(bagFlowGrade) == false) {
                List<GradeVO> gradeList = JSON.parseArray(bagFlowGrade, GradeVO.class);
                for (GradeVO gradeVO : gradeList) {
                    int start = gradeVO.getStart();
                    int value = Optional.ofNullable(gradeVO.getValue()).orElse(0);
                    switch (start) {
                        case 0:
                            addGradeMapTotal(gradeMap, 1014L, Long.valueOf(value));
                            break;
                        case 100:
                            addGradeMapTotal(gradeMap, 1015L, Long.valueOf(value));
                            break;
                        case 500:
                            addGradeMapTotal(gradeMap, 1016L, Long.valueOf(value));
                            break;
                        case 1000:
                            addGradeMapTotal(gradeMap, 1017L, Long.valueOf(value));
                            break;
                        case 3000:
                            addGradeMapTotal(gradeMap, 1018L, Long.valueOf(value));
                            break;
                        default:
                            break;
                    }
                }
            }

            Long directFlowTotal = Optional.ofNullable(gradeDO.getDirectNormalFlow()).orElse(0L);
            Long directLuckTotal = Optional.ofNullable(gradeDO.getDirectLuckFlow()).orElse(0L);
            Long bagFlowTotal = Optional.ofNullable(gradeDO.getBagFlow()).orElse(0L);

            addGradeMapTotal(gradeMap, 1019L, directFlowTotal);
            addGradeMapTotal(gradeMap, 1020L, directLuckTotal);
            addGradeMapTotal(gradeMap, 1021L, bagFlowTotal);
        }

        if (list.get(0).getDimension() == 0) {
            log.info("计算房间评分：totalMap:" + JSON.toJSONString(gradeMap));
        }

        Map<Integer, GradeConfigDO> gradeConfigDOMap = gradeConfigDAO.selectConfigMap(0);
        int weightTotal = gradeConfigDOMap.values().stream().filter(k -> k.getWeight() != null).mapToInt(GradeConfigDO::getWeight).sum();
        for (StatRtcRoomGradeDO statRtcRoomGradeDO : list) {
            Double score = 0.0;

            Map<Long, Long> valueMap = new HashMap<>();
            valueMap.put(1001L, Optional.ofNullable(statRtcRoomGradeDO.getActivityUserCount()).orElse(0L));
            valueMap.put(1002L, Optional.ofNullable(statRtcRoomGradeDO.getFavoriteUserCount()).orElse(0L));
            valueMap.put(1003L, Optional.ofNullable(statRtcRoomGradeDO.getSitSeatUserCount()).orElse(0L));
            valueMap.put(1004L, Optional.ofNullable(statRtcRoomGradeDO.getPaidUserCount()).orElse(0L));
            valueMap.put(1005L, Optional.ofNullable(statRtcRoomGradeDO.getSpeakNum()).orElse(0L));
            valueMap.put(1006L, Optional.ofNullable(statRtcRoomGradeDO.getBarrageNum()).orElse(0L));
            valueMap.put(1007L, Optional.ofNullable(statRtcRoomGradeDO.getHighestOnlineUserCount()).orElse(0L));
            valueMap.put(1008L, Optional.ofNullable(statRtcRoomGradeDO.getUserStayTimes()).orElse(0L));

            //直刷档次
            String directFlowJSON = statRtcRoomGradeDO.getDirectFlowGrade();
            if (StringUtils.isNotBlank(directFlowJSON) && ("null".equals(directFlowJSON) == false)) {
                List<GradeVO> gradeList = JSON.parseArray(directFlowJSON, GradeVO.class);
                for (GradeVO gradeVO : gradeList) {
                    int start = gradeVO.getStart();
                    Long value = Optional.ofNullable(gradeVO.getValue()).orElse(0).longValue();
                    switch (start) {
                        case 0:
                            valueMap.put(1009L, value);
                            break;
                        case 100:
                            valueMap.put(1010L, value);
                            break;
                        case 500:
                            valueMap.put(1011L, value);
                            break;
                        case 1000:
                            valueMap.put(1012L, value);
                            break;
                        case 3000:
                            valueMap.put(1013L, value);
                            break;
                        default:
                            break;
                    }
                }
            }
            //背包档次
            String bagFlowJSON = statRtcRoomGradeDO.getBagFlowGrade();
            if (StringUtils.isNotBlank(bagFlowJSON) && ("null".equals(bagFlowJSON) == false)) {
                List<GradeVO> gradeList = JSON.parseArray(bagFlowJSON, GradeVO.class);
                for (GradeVO gradeVO : gradeList) {
                    int start = gradeVO.getStart();
                    Long value = Optional.ofNullable(gradeVO.getValue()).orElse(0).longValue();
                    switch (start) {
                        case 0:
                            valueMap.put(1014L, value);
                            break;
                        case 100:
                            valueMap.put(1015L, value);
                            break;
                        case 500:
                            valueMap.put(1016L, value);
                            break;
                        case 1000:
                            valueMap.put(1017L, value);
                            break;
                        case 3000:
                            valueMap.put(1018L, value);
                            break;
                        default:
                            break;
                    }
                }
            }

            Long directFlowValue = Optional.ofNullable(statRtcRoomGradeDO.getDirectNormalFlow()).orElse(0L);
            Long directLuckFlowValue = Optional.ofNullable(statRtcRoomGradeDO.getDirectLuckFlow()).orElse(0L);
            Long bagFlowValue = Optional.ofNullable(statRtcRoomGradeDO.getBagFlow()).orElse(0L);
            valueMap.put(1019L, directFlowValue);
            valueMap.put(1020L, directLuckFlowValue);
            valueMap.put(1021L, bagFlowValue);

            for (Map.Entry<Integer, GradeConfigDO> entry : gradeConfigDOMap.entrySet()) {
                Integer weight = entry.getValue().getWeight();
                if (weight <= 0) {
                    continue;
                }

                Long tag = entry.getKey().longValue();
                if (valueMap.get(tag) != null) {
                    long totalValue = gradeMap.get(tag).longValue();
                    long countValue = Optional.ofNullable(valueMap.get(tag)).orElse(0L).longValue();
                    if (statRtcRoomGradeDO.getDimension() == 0 && statRtcRoomGradeDO.getRoomId() == 518732426248192L) {
                        log.info("计算房间评分：tag:" + tag + " totalvalue:" + totalValue + "  countValue:" + countValue);
                    }
                    if (totalValue == 0 || countValue == 0) {
                        continue;
                    }
                    score += (countValue / (totalValue * 1.0)) * weight;
                }
            }

            if (statRtcRoomGradeDO.getDimension() == 0 && statRtcRoomGradeDO.getRoomId() == 518732426248192L) {
                log.info("计算房间评分：valueMap:" + JSON.toJSONString(valueMap));
                log.info("计算房间评分：score:" + score);
                log.info("计算房间评分：weightTotal:" + weightTotal);
                log.info("计算房间评分：result:" + score * weightTotal);
            }

            statRtcRoomGradeDO.setScore(score * weightTotal);
        }
    }

    /**
     * 默认gradeVOS列表
     *
     * @return
     */
    private List<GradeVO> defaultGradeVOList() {
        List<GradeVO> gradeVOList = new ArrayList<>();
        gradeVOS.forEach(k -> {
            GradeVO tempVO = new GradeVO();
            BeanUtils.copyProperties(k, tempVO);
            gradeVOList.add(tempVO);
        });

        return gradeVOList;
    }

    private Map<Long, Long> getDefaultGradeMap() {
        Map<Long, Long> gradeMap = new HashMap<>();
        for (int index = 1001; index <= 1021; index++) {
            gradeMap.put(Long.valueOf(index), 0L);
        }

        return gradeMap;
    }

    private void addGradeMapTotal(Map<Long, Long> defaultGradeMap, Long tag, Long value) {
        if (defaultGradeMap == null || tag == null || value == null) {
            log.info("默认档次层级字典修改参数为空");
            return;
        }
        Long total = Optional.ofNullable(defaultGradeMap.get(tag)).orElse(0L) + value;
        defaultGradeMap.put(tag, total);
    }

}
