package com.starbuds.server.service.task.job;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.common.constant.message.im.ImTargetType;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.message.im.rtc.AppWidgetMsg;
import com.starbuds.server.common.pojo.message.im.rtc.RtcFireworkMsg;
import com.starbuds.server.common.service.data.ReleaseConfigService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

/***
 * @author qinjiang
 * @description 计时任务(时间相关处理)
 * @date 2021/12/11
 */
@Component
@Slf4j
public class TimekeeperJob {

    @Reference(version = "1.0.0")
    ReleaseConfigService releaseConfigService;

    @Reference(version = "1.0.0")
    ImService imService;

    @XxlJob(value = JobHandlers.FIREWORK_NOTIFY_TASK_HANDER)
    public ReturnT fireworkNotifyTaskHandler(String param) {
        String fireworkConfigId = releaseConfigService.getReleaseConfig("firework");
        if (StringUtils.isBlank(fireworkConfigId)) {
            log.info("-- ZERO_NOTIFY_TASK_HANDER：烟花参数配置错误");
            return ReturnT.FAIL;
        }
        Long fireworkId = StringUtils.isNumeric(fireworkConfigId) ? Long.valueOf(fireworkConfigId) : 0L;

        RtcFireworkMsg fireworkMsg = new RtcFireworkMsg();
        fireworkMsg.setFireworkId(fireworkId);
        fireworkMsg.setRoomId(0L);

        log.info("--ZERO_NOTIFY_TASK_HANDER：：: imMsg:{}", JSON.toJSONString(fireworkMsg));

        imService.sendRtcRoomMsg(ImTargetType.ALL, null, null, null, fireworkMsg);

        return ReturnT.SUCCESS;
    }

    /**
     * 用于零点进行通知挂件是否开启或者关闭
     *
     * @param param
     * @return
     */
    @XxlJob(value = JobHandlers.WIDGET_NOTIFY_TASK_HANDER)
    public ReturnT widgetNotifyTaskHander(String param) {
        String widgetConfig = releaseConfigService.getReleaseConfig("widget");
        if (StringUtils.isBlank(widgetConfig)) {
            log.info("--WIDGET_NOTIFY_TASK_HANDER：挂件参数配置错误");
            return ReturnT.FAIL;
        }

        long currentTimestamp = System.currentTimeMillis();
        JSONObject widgetJSON = JSON.parseObject(widgetConfig);
        boolean display = currentTimestamp > widgetJSON.getLongValue("st") && currentTimestamp < widgetJSON.getLongValue("et");

        AppWidgetMsg appWidgetMsg = new AppWidgetMsg();
        appWidgetMsg.setIsDisplay(display);
        imService.sendRtcRoomMsg(ImTargetType.ALL, null, null, null, appWidgetMsg);

        return ReturnT.SUCCESS;
    }
}
