package com.starbuds.server.service.task.job;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.common.pojo.db.UserNoSpeakHisDO;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.utils.DateUtil;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.UserNoSpeakHisDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * 新增定时任务解封用户禁言
 *
 * @Author: lujiao
 * @Date: 2021/9/10 10:33
 */
@Slf4j
@Component
public class UserNoSpeakFreeJob {

    @Resource
    UserNoSpeakHisDAO userNoSpeakHisDAO;

    @Reference(version = "1.0.0")
    ImService imService;


    @XxlJob(value = JobHandlers.USER_NO_SPEAK_FREE_TASK)
    public ReturnT<String> userNoSpeakFreeHandler(String param) {
        Long now = System.currentTimeMillis();
        Long yesterday = now - DateUtil.DAY_OF_MILLION;
        List<UserNoSpeakHisDO> userNoSpeakHisDOList = userNoSpeakHisDAO.getNeedFreeList(yesterday, now);
        if (CollectionUtil.isNotEmpty(userNoSpeakHisDOList)) {
            userNoSpeakHisDOList.forEach(i -> {
                doUserNoSpeakFree(i);
            });
        }
        log.info("用户解禁定时任务完成");
        return ReturnT.SUCCESS;
    }

    @Transactional(rollbackFor = Exception.class)
    public void doUserNoSpeakFree(UserNoSpeakHisDO hisDO) {
        boolean cancelResult = imService.cancelForbiddenWords(hisDO.getUserId());
        if (cancelResult) {
            userNoSpeakHisDAO.updateSetHandle(hisDO.getHisId(), System.currentTimeMillis());
        }
    }

}
