package com.starbuds.server.service.task.job;

import com.starbuds.server.common.service.user.UserWatchRelationService;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.stereotype.Component;

/**
 * 守护的Job
 *
 * @author: Mazexal
 * Date: 2021/7/26
 */
@Slf4j
@Component
public class WatchJob {

    @Reference(version = "1.0.0")
    UserWatchRelationService watchService;


    /**
     * 用于新增守护到期提醒记录
     *
     * @param param
     * @return
     */
    @XxlJob(value = JobHandlers.BUILD_WATCH_NOTIFY)
    public ReturnT<String> buildWatchNotifyHandler(String param) {
        log.info("开始生成守护到期提醒记录");
        watchService.buildWatchNotify();
        return ReturnT.SUCCESS;
    }


    /**
     * 记录守护统计Top20, 并把胜利者进行记录
     *
     * @param param
     * @return
     */
    @XxlJob(value = JobHandlers.WATCH_SCORE_RANK)
    public ReturnT<String> watchScoreRankHandler(String param) {
        log.info("固定守护守护统计Top20");
        watchService.buildWatchIntimacyTopHis();
        return ReturnT.SUCCESS;
    }
}
