package com.starbuds.server.service.task.job;

import com.starbuds.server.common.constant.finance.WalletTypeEnum;
import com.starbuds.server.common.pojo.task.vo.StatFinancialWeeklyVO;
import com.starbuds.server.service.task.constant.JobHandlers;
import com.starbuds.server.service.task.dao.StatFinancialWeeklyDAO;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * 财务周粉钻统计
 *
 * @author: ruomu
 * @date: 2021/11/29 created
 */

@Slf4j
@Component
public class statFinancialWeeklyJob {

    @Resource
    StatFinancialWeeklyDAO statFinancialWeeklyDAO;

    private static final int startYear = 2021;

    /**
     * @param param yyyy-MM-dd 年月日 2020-02-01
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @XxlJob(value = JobHandlers.STAT_FINANCIAL_WEEKLY)
    public ReturnT<String> statUserHandler(String param) {
        try {
            log.info("财务周统计 {}", param);
            //有参数
            if (StringUtils.isNotEmpty(param)) {
                //传入的时间处理，获取年，获取日期所在的周,获取周的开始时间和结束时间戳
                SimpleDateFormat sm = new SimpleDateFormat("yyyy-MM-dd");
                Date d = sm.parse(param);
                int year = Integer.parseInt(String.format("%tY", d));

                String[] yearandweek = getWeek(param).split(",");
                int yearNew = Integer.parseInt(yearandweek[0]);
                int week = Integer.parseInt(yearandweek[1]);

                String weekStartAndEnd = getWeekDays(yearNew, week, 0);
                String[] ws = weekStartAndEnd.split(",");
                SimpleDateFormat wsSdf = new SimpleDateFormat("yyyy-MM-dd");

                long start = wsSdf.parse(ws[0]).getTime();
                long end = wsSdf.parse(ws[1]).getTime();

                //统计
                stat(year, week, start, end);


            } else {
                //无参数统计当前周的上一周
                //获取年，获取日期所在的周,获取周的开始时间和结束时间戳
                Calendar cal = Calendar.getInstance();
                int year = cal.get(Calendar.YEAR);

                //获取当前日期的上周是第几周
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                String now = sdf.format(new Date());
                String[] yearandweek = getWeek(now).split(",");
                int yearNew = Integer.parseInt(yearandweek[0]);
                int week = Integer.parseInt(yearandweek[1]);

                //上一周的开始是按和结束时间
                String weekStartAndEnd = getWeekDays(yearNew, week, 0);
                String[] ws = weekStartAndEnd.split(",");
                SimpleDateFormat wsSdf = new SimpleDateFormat("yyyy-MM-dd");
                //开始和结束时间
                long start = wsSdf.parse(ws[0]).getTime();
                long end = wsSdf.parse(ws[1]).getTime();
                stat(year, week, start, end);
            }
        } catch (Exception ex) {
            log.error("财务周统计错误:", ex);
            return ReturnT.FAIL;
        }

        return ReturnT.SUCCESS;
    }


    private void stat(int year, int week, long start, long end) throws ParseException {

        week = (year - startYear) * 52 + week;

        log.info("statFincialWeekl" + "year:" + year + "week:" + week + "start:" + start + "end:" + end);
        int typeDisable = WalletTypeEnum.STAR_DUST.getValue();
        int typeEnable = WalletTypeEnum.PINK_DIAMOND.getValue();

        StatFinancialWeeklyVO stat = statFinancialWeeklyDAO.getStatFinancial(year, week);
        if (stat == null || stat.getStatHisId() <= 0) {
            //不存在数据插入
            StatFinancialWeeklyVO mod = new StatFinancialWeeklyVO();

            mod.setYear(year);
            mod.setWeek(week);
            mod.setStartTime(start);
            mod.setEndTime(end);
            mod.setCreateTime(System.currentTimeMillis());
            mod.setUpdateTime(System.currentTimeMillis());

            //周上期结余 粉钻（可提现 和 不可提现）
            mod.setPDiamondDisable(check(statFinancialWeeklyDAO.getpDiamond(typeDisable, start)));
            mod.setPDiamondEnable(check(statFinancialWeeklyDAO.getpDiamond(typeEnable, start)));

            //周变化粉钻 （可提现 和 不可提现）
            mod.setChangePDiamondDisable(check(statFinancialWeeklyDAO.changePDiamond(typeDisable, start, end)));
            mod.setChangePDiamondEnable(check(statFinancialWeeklyDAO.changePDiamond(typeEnable, start, end)));

            //期末结余粉钻（可提现 和 不可提现）
            mod.setOverPDiamondDisable(check(statFinancialWeeklyDAO.overPDiamond(typeDisable, end)));
            mod.setOverPDiamondEnable(check(statFinancialWeeklyDAO.overPDiamond(typeEnable, end)));

            //增加的不可提现粉钻数量
            mod.setAddPDiamondDisable(check(statFinancialWeeklyDAO.getAddPDiamondDisable(start, end)));

            //粉钻兑换钻石消耗粉钻数量（可提现 和 不可提现）
            mod.setPDisableToDiamond(check(statFinancialWeeklyDAO.pDiamondToDiamond(typeDisable, start, end)));
            mod.setPEnableToDiamond(check(statFinancialWeeklyDAO.pDiamondToDiamond(typeEnable, start, end)));

            //可提现粉钻后台扣除粉钻数量
            mod.setPEnableBackDeduction(check(statFinancialWeeklyDAO.pEnableBackDeduction(start, end)));

            //可提现粉钻提现数量
            mod.setPEnableWithdrawal(check(statFinancialWeeklyDAO.pEnableWithdrawal(start, end)));

            //周内审核通过总提现粉钻数量
            long applePDiamond = check(statFinancialWeeklyDAO.applePDiamond(start, end));
            mod.setApplePDiamond(applePDiamond);

            //周内审核通过且周内申请提现粉钻总量
            long appleCreatePDiamond = check(statFinancialWeeklyDAO.appleCreatePDiamond(start, end));
            mod.setAppleCreatePDiamond(appleCreatePDiamond);

            //周内审核通过非周内申请体现粉钻数量
            mod.setAppleNotCreatePDiamond(applePDiamond - appleCreatePDiamond);

            statFinancialWeeklyDAO.insert(mod);
        } else {
            log.info("statyw:" + stat.getStatHisId() + "," + stat.getYear() + "," + stat.getWeek());
            //存在数据更新
            StatFinancialWeeklyVO mod = new StatFinancialWeeklyVO();
            mod.setStatHisId(stat.getStatHisId());
            mod.setYear(year);
            mod.setWeek(week);
            mod.setStartTime(start);
            mod.setEndTime(end);
            mod.setCreateTime(System.currentTimeMillis());
            mod.setUpdateTime(System.currentTimeMillis());

            //周上期结余 粉钻（可提现 和 不可提现）
            mod.setPDiamondDisable(check(statFinancialWeeklyDAO.getpDiamond(typeDisable, start)));
            mod.setPDiamondEnable(check(statFinancialWeeklyDAO.getpDiamond(typeEnable, start)));

            //周变化粉钻 （可提现 和 不可提现）
            mod.setChangePDiamondDisable(check(statFinancialWeeklyDAO.changePDiamond(typeDisable, start, end)));
            mod.setChangePDiamondEnable(check(statFinancialWeeklyDAO.changePDiamond(typeEnable, start, end)));

            //期末结余粉钻（可提现 和 不可提现）
            mod.setOverPDiamondDisable(check(statFinancialWeeklyDAO.overPDiamond(typeDisable, end)));
            mod.setOverPDiamondEnable(check(statFinancialWeeklyDAO.overPDiamond(typeEnable, end)));

            //增加的不可提现粉钻数量
            mod.setAddPDiamondDisable(check(statFinancialWeeklyDAO.getAddPDiamondDisable(start, end)));

            //粉钻兑换钻石消耗粉钻数量（可提现 和 不可提现）
            mod.setPDisableToDiamond(check(statFinancialWeeklyDAO.pDiamondToDiamond(typeDisable, start, end)));
            mod.setPEnableToDiamond(check(statFinancialWeeklyDAO.pDiamondToDiamond(typeEnable, start, end)));

            //可提现粉钻后台扣除粉钻数量
            mod.setPEnableBackDeduction(check(statFinancialWeeklyDAO.pEnableBackDeduction(start, end)));

            //可提现粉钻提现数量
            mod.setPEnableWithdrawal(check(statFinancialWeeklyDAO.pEnableWithdrawal(start, end)));

            //周内审核通过总提现粉钻数量
            long applePDiamond = check(statFinancialWeeklyDAO.applePDiamond(start, end));
            mod.setApplePDiamond(applePDiamond);

            //周内审核通过且周内申请提现粉钻总量
            long appleCreatePDiamond = check(statFinancialWeeklyDAO.appleCreatePDiamond(start, end));
            mod.setAppleCreatePDiamond(appleCreatePDiamond);

            //周内审核通过非周内申请体现粉钻数量
            mod.setAppleNotCreatePDiamond(applePDiamond - appleCreatePDiamond);

            statFinancialWeeklyDAO.update(mod);
        }
    }


    /**
     * 根据具体年份周数获取日期范围
     *
     * @param year
     * @param week
     * @param targetNum
     * @return 2021-01-11,2021-01-17
     */
    public static String getWeekDays(int year, int week, int targetNum) {
        // 计算目标周数
        if (week + targetNum > 52) {
            year++;
            week += targetNum - 52;
        } else if (week + targetNum <= 0) {
            year--;
            week += targetNum + 52;
        } else {
            week += targetNum;
        }

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Calendar cal = Calendar.getInstance();

        // 设置每周的开始日期
        cal.setFirstDayOfWeek(Calendar.MONDAY);

        cal.set(Calendar.YEAR, year);
        cal.set(Calendar.WEEK_OF_YEAR, week);
        String yearT = sdf.format(cal.getTime());

        cal.set(Calendar.DAY_OF_WEEK, cal.getFirstDayOfWeek());
        String beginDate = sdf.format(cal.getTime());

        cal.add(Calendar.DAY_OF_WEEK, 6);
        String endDate = sdf.format(cal.getTime());


        return beginDate + "," + endDate + "," + week;

    }


    /**
     * 获取周
     *
     * @param yearStr 2012-12-31
     * @return
     */
    /**
     * @param yearStr 2012-12-31
     * @return
     */
    public static String getWeek(String yearStr) throws ParseException {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Calendar cl = Calendar.getInstance();
        cl.setTime(sdf.parse(yearStr));

        int oYear = cl.get(Calendar.YEAR);
        cl.add(Calendar.DAY_OF_MONTH, -7);
        int week = cl.get(Calendar.WEEK_OF_YEAR);
        int year = cl.get(Calendar.YEAR);


        if (week < cl.get(Calendar.WEEK_OF_YEAR)) {
            year += 1;
        }

        if (year != oYear) {
            week = week + 52;
        }

        System.out.println(year + "年第" + week + "周");
        return year + "," + week;
    }


    private Long check(Long p) {
        return p == null ? 0L : p;
    }


}
