package com.starbuds.server.service.task.provider;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.db.StatGoldCoinExchangeDO;
import com.starbuds.server.service.task.dao.StatGoldCoinExchangeDAO;
import com.starbuds.server.common.pojo.task.qo.StatGoldCoinExchangePageListQO;

import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.pojo.api.PageList;
import lombok.extern.slf4j.Slf4j;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.starbuds.server.common.pojo.daoplus.UpdateWrapper;
import com.starbuds.server.common.pojo.daoplus.QueryWrapper;
import com.starbuds.server.common.service.core.IdentityService;

import static com.starbuds.server.common.pojo.util.PageListUtil.buildPageListQueryWrapper;

import org.springframework.stereotype.Component;
import org.apache.dubbo.config.annotation.Reference;
import org.springframework.transaction.annotation.Transactional;

/**
 * 金币兑换物品统计
 *
 * @author aoe
 */
@Component
@Slf4j
public class StatGoldCoinExchangeProvider {

    /**
     * 主键
     */
    private String PRIMARY_KEY = "exchangeId";

    @Reference(version = "1.0.0")
    private IdentityService identityService;

    @Resource
    private StatGoldCoinExchangeDAO statGoldCoinExchangeDAO;

    /**
     * 插入数据
     *
     * @param po
     * @return null:插入失败
     */
    @Transactional(rollbackFor = Exception.class)
    public StatGoldCoinExchangeDO create(StatGoldCoinExchangeDO po) {
        po.setExchangeId(identityService.getId());
        po.setCreateTime(System.currentTimeMillis());
        po.setUpdateTime(po.getCreateTime());
        po.setIsDeleted(BoolIntVal.False);
        boolean flag = statGoldCoinExchangeDAO.insert(po) == 1;

        String json = JSON.toJSONString(po);
        if (flag) {
            log.info("金币兑换物品统计创建成功 {}", json);
            return po;
        }
        log.warn("金币兑换物品统计创建失败 {}", json);
        return null;
    }

    /**
     * 插入批量数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createBatch(List<StatGoldCoinExchangeDO> list) {
        boolean flag = statGoldCoinExchangeDAO.insertBatch(list) > 0;
        return flag;
    }

    /**
     * 插入或更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean createOrUpdate(StatGoldCoinExchangeDO po) {
        boolean flag = statGoldCoinExchangeDAO.insertOrUpdate(po) == 1;
        return flag;
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(StatGoldCoinExchangeDO po) {
        po.setCreateTime(null);
        po.setUpdateTime(System.currentTimeMillis());

        UpdateWrapper uw = new UpdateWrapper();
        uw.setPo(po);
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, po.getExchangeId());
        uw.setQw(qw);
        return update(uw);
    }

    /**
     * 更新数据
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean update(UpdateWrapper uw) {
        boolean flag = statGoldCoinExchangeDAO.update(uw) > 0;
        return flag;
    }

    /**
     * 根据id查询
     */
    public StatGoldCoinExchangeDO getById(Long id) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return statGoldCoinExchangeDAO.selectOne(qw);
    }

    /**
     * 判断是否存在
     *
     * @param id 主键
     * @return true:存在
     */
    public boolean exist(Long id) {
        if (id == null) {
            return false;
        }
        QueryWrapper qw = new QueryWrapper();
        qw.eq(PRIMARY_KEY, id);
        return statGoldCoinExchangeDAO.selectCount(qw) > 0;
    }

    /**
     * 分页查询
     */
    public PageList<StatGoldCoinExchangeDO> getPageList(StatGoldCoinExchangePageListQO qo) {
        QueryWrapper qw = buildPageListQueryWrapper(qo);
        PageList<StatGoldCoinExchangeDO> page = new PageList<>(qo.getPageIndex(), qo.getPageSize());
        qw.setPageCursor(page.getPageCursor());
        qw.setPageSize(page.getPageSize());
        page.setList(statGoldCoinExchangeDAO.selectPageList(qw));
        page.setTotal(statGoldCoinExchangeDAO.selectCount(qw));
        return page;
    }

    /**
     * 根据ids获得 DO Map
     *
     * @param ids id集合
     * @return Map<主键, DO对象>
     */
    public Map<Long, StatGoldCoinExchangeDO> getMapByIds(Collection ids) {
        return statGoldCoinExchangeDAO.selectMapByIds(ids);
    }

    // 手动编码开始

}