package com.starbuds.server.service.task.util;

import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

/**
 * @Author lujiao
 * @DateTime 2021/7/8 18:07
 * @Description
 */
@Slf4j
@Component
public class DingDingUtil {

    @Value("${dingding.secret}")
    String secret;

    @Value("${dingding.dingDingUrl}")
    String dingDingUrl;

    /**
     * 发送钉钉消息
     *
     * @param content
     */
    public void sendDingDingWarn(String content) {
        String dingDingToken = dingDingUrl;
        long timeStamp = System.currentTimeMillis();
        try {
            dingDingToken = dingDingToken + "&timestamp=" + timeStamp + "&sign=" + sing(timeStamp);
            Map<String, Object> json = new HashMap(3);
            Map<String, Object> text = new HashMap(3);
            json.put("msgtype", "text");
            text.put("content", content);
            json.put("text", text);
            String res = HttpUtil.post(dingDingToken, JSON.toJSONString(json));
            log.info("钉钉消息发送结果:" + res);
        } catch (Exception e) {
            log.error("钉钉消息发送失败", e);
        }
    }

    /**
     * 钉钉签名
     *
     * @param timeStamp
     * @return
     * @throws Exception
     */
    private String sing(Long timeStamp) throws Exception {
        String stringToSign = timeStamp + "\n" + secret;
        Mac mac = Mac.getInstance("HmacSHA256");
        mac.init(new SecretKeySpec(secret.getBytes("UTF-8"), "HmacSHA256"));
        byte[] signData = mac.doFinal(stringToSign.getBytes("UTF-8"));
        return new String(Base64.getEncoder().encode(signData));
    }
}
