package com.starbuds.server.service.task.job;

import com.starbuds.server.common.constant.activity.ActivityTypeEnum;
import com.starbuds.server.common.constant.activity.RankTypeEnum;
import com.starbuds.server.common.constant.cache.RedisCacheKeys;
import com.starbuds.server.common.pojo.activity.ActivityVO;
import com.starbuds.server.common.service.activity.ActivityService;
import com.starbuds.server.common.service.core.CacheService;
import com.starbuds.test.BaseTest;
import com.xxl.job.core.biz.model.ReturnT;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.junit.Assert;
import org.junit.jupiter.api.Test;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.*;

/**
 * 活动排名记录持久化测试
 */
@Slf4j
public class ActivityRankPersistenceJobTest extends BaseTest {

    @Resource
    private ActivityRankPersistenceJob activityRankPersistenceJob;

    @Reference(version = "1.0.0")
    private ActivityService activityService;

    @Reference(version = "1.0.0")
    private CacheService cacheService;

    @Test
    public void lastRecordJobHandlerTest() throws InterruptedException {
        // PK 活动 2020-05-09
        int activityTypePK = ActivityTypeEnum.PK.getValue();
        ActivityVO activity = activityService.getActivityByType(activityTypePK);
        Long activityId = activity.getActivityId();

        List<String> keyList = new ArrayList<>(RankTypeEnum.values().length);
        int count = 10, threads = 1;
        ExecutorService fixedThreadPool = Executors.newFixedThreadPool(threads);
        for (RankTypeEnum rankTypeEnum : RankTypeEnum.values()) {
            String key = RedisCacheKeys.getActivityRankKey(activityId, rankTypeEnum.getValue());
            keyList.add(key);
            log.info("需要删除的key {}", key);
            final CountDownLatch latch = new CountDownLatch(count);
            for (int i = 0; i < count; i++) {
                final int index = i;
                fixedThreadPool.execute(() -> {
                    cacheService.zAdd(key, Integer.valueOf(100 + index).toString(), Double.valueOf(index));
                    latch.countDown();
                });
            }
            latch.await();
        }

        Assert.assertEquals(activityRankPersistenceJob.lastRecordJobHandler("" + activityTypePK), ReturnT.SUCCESS);
        for (String key : keyList) {
            String value = cacheService.get(key);
            System.out.println(key + "  " + value);
            Assert.assertEquals(cacheService.hasKey(key), Boolean.FALSE);
        }

        Assert.assertEquals(activityRankPersistenceJob.lastRecordJobHandler(null), ReturnT.SUCCESS);
        Assert.assertEquals(activityRankPersistenceJob.lastRecordJobHandler("a"), ReturnT.SUCCESS);
        Assert.assertEquals(activityRankPersistenceJob.lastRecordJobHandler("1"), ReturnT.SUCCESS);
    }

}